@@ -45,6 +45,13 @@ __version__ = version_string
 
 bzrlib.api.require_any_api(bzrlib, bzr_compatible_versions)
 
+try:
+    from bzrlib.i18n import load_plugin_translations
+except ImportError: # No translations for bzr < 2.5
+    gettext = lambda x: x
+else:
+    translation = load_plugin_translations("bzr-svn")
+    gettext = translation.gettext
 
 from bzrlib import config
 from bzrlib.branch import (
@@ -85,6 +92,7 @@ from bzrlib.revisionspec import (
     revspec_registry,
     )
 from bzrlib.transport import (
+    ConnectedTransport,
     register_lazy_transport,
     register_transport_proto,
     )
@@ -158,14 +166,15 @@ class SvnWorkingTreeProber(SvnProber):
     def probe_transport(self, transport):
         from bzrlib.transport.local import LocalTransport
 
-        if isinstance(transport, LocalTransport) and transport.has(".svn"):
-            self._check_versions()
-            from subvertpy.wc import check_wc
-            version = check_wc(transport.local_abspath('.').encode("utf-8"))
-            from bzrlib.plugins.svn.workingtree import SvnWorkingTreeDirFormat
-            return SvnWorkingTreeDirFormat(version)
+        if (not isinstance(transport, LocalTransport)
+            or not transport.has(".svn")):
+            raise NotBranchError(path=transport.base)
 
-        raise NotBranchError(path=transport.base)
+        self._check_versions()
+        from subvertpy.wc import check_wc
+        version = check_wc(transport.local_abspath('.').encode("utf-8"))
+        from bzrlib.plugins.svn.workingtree import SvnWorkingTreeDirFormat
+        return SvnWorkingTreeDirFormat(version)
 
     @classmethod
     def known_formats(cls):
@@ -199,7 +208,6 @@ def dav_options(transport, url):
                 conn.setopt(pycurl.URL, url)
                 transport._set_curl_options(conn)
                 conn.setopt(pycurl.CUSTOMREQUEST, 'OPTIONS')
-                conn.setopt(pycurl.NOBODY, 1)
                 header = StringIO()
                 data = StringIO()
                 conn.setopt(pycurl.HEADERFUNCTION, header.write)
@@ -221,9 +229,17 @@ class SvnRemoteProber(SvnProber):
     _supported_schemes = ["http", "https", "file", "svn"]
 
     def probe_transport(self, transport):
-        from bzrlib.transport.local import LocalTransport
 
-        if isinstance(transport, LocalTransport):
+        try:
+            url = transport.external_url()
+        except InProcessTransport:
+            # bzr-svn not supported on MemoryTransport
+            raise NotBranchError(path=transport.base)
+
+        if url.startswith("readonly+"):
+            url = url[len("readonly+"):]
+
+        if url.startswith("file://"):
             # Cheaper way to figure out if there is a svn repo
             maybe = False
             subtransport = transport
@@ -241,11 +257,7 @@ class SvnRemoteProber(SvnProber):
             if not maybe:
                 raise NotBranchError(path=transport.base)
 
-        try:
-            scheme = transport.external_url().split(":")[0]
-        except InProcessTransport:
-            # bzr-svn not supported on MemoryTransport
-            raise NotBranchError(path=transport.base)
+        scheme = url.split(":")[0]
         if (not scheme.startswith("svn+") and
             not scheme in self._supported_schemes):
             raise NotBranchError(path=transport.base)
@@ -253,8 +265,11 @@ class SvnRemoteProber(SvnProber):
         # If this is a HTTP transport, use the existing connection to check
         # that the remote end supports version control.
         if scheme in ("http", "https"):
-            url = transport._unsplit_url(transport._unqualified_scheme,
-                None, None, transport._host, transport._port, transport._path)
+            priv_transport = getattr(transport, "_decorated", transport)
+            url = ConnectedTransport._unsplit_url(
+                priv_transport._unqualified_scheme,
+                None, None, priv_transport._host,
+                priv_transport._port, priv_transport._path)
             try:
                 dav_entries = dav_options(transport, url)
             except (InProcessTransport, NoSuchFile, InvalidURL, InvalidHttpResponse):
