@@ -16,6 +16,7 @@
 
 """Annotate support."""
 
+import subvertpy
 
 from cStringIO import (
     StringIO,
@@ -24,9 +25,16 @@ from subvertpy.delta import (
     apply_txdelta_window,
     )
 
+from bzrlib import (
+    trace,
+    urlutils,
+    )
 from bzrlib.annotate import (
     reannotate,
     )
+from bzrlib.errors import (
+    NoSuchId,
+    )
 
 from bzrlib.plugins.svn import (
     changes,
@@ -35,27 +43,39 @@ from bzrlib.plugins.svn import (
 
 class Annotater(object):
 
-    def __init__(self, repository, mapping, fileid, revid):
+    def __init__(self, repository, fileid):
         self._text = ""
         self._annotated = []
-        self._mapping = mapping
         self._repository = repository
         self.fileid = fileid
         self._related_revs = {}
-        (_, branch_path, revnum), mapping = self._repository.lookup_bzr_revision_id(
-            revid)
-        # FIXME: Cope with restarts
-        for (revmeta, mapping) in self._repository._revmeta_provider._iter_reverse_revmeta_mapping_history(branch_path, revnum,
-                to_revnum=0, mapping=mapping):
-            self._related_revs[revmeta.metarev.revnum] = revmeta, mapping
 
     def get_annotated(self):
         return self._annotated
 
-    def check_file_revs(self, path, revnum):
+    def check_file_revs(self, revid, branch_path, revnum, mapping, relpath=None):
+        self._annotated = []
+        for (revmeta, hidden, mapping) in self._repository._revmeta_provider._iter_reverse_revmeta_mapping_history(
+                branch_path, revnum, to_revnum=0, mapping=mapping):
+            if hidden:
+                continue
+            self._related_revs[revmeta.metarev.revnum] = revmeta, mapping
         self._text = ""
-        self._repository.transport.get_file_revs(path, -1, revnum,
-            self._handler)
+        if relpath is None:
+            tree = self._repository.revision_tree(revid)
+            try:
+                relpath = tree.id2path(self.fileid)
+            except NoSuchId:
+                return []
+        path = urlutils.join(branch_path, relpath.encode("utf-8")).strip("/")
+        try:
+            self._repository.transport.get_file_revs(path, -1, revnum,
+                self._handler, include_merged_revisions=True)
+        except subvertpy.SubversionException, (msg, num):
+            if num == subvertpy.ERR_FS_NOT_FILE:
+                return []
+            raise
+        return self._annotated
 
     def _get_ids(self, path, rev, revprops):
         revmeta, mapping = self._related_revs[rev]
@@ -67,6 +87,10 @@ class Annotater(object):
         idmap = self._repository.get_fileid_map(revmeta, mapping)
         return idmap.lookup(mapping, ip)
 
+    def check_file(self, lines, revid, parent_lines):
+        trace.mutter('annotate %r, %r', self.fileid, revid)
+        return reannotate(parent_lines, lines, revid)
+
     def _handler(self, path, rev, revprops, result_of_merge=None):
         try:
             fileid, revid, _ = self._get_ids(path, rev, revprops)
@@ -82,9 +106,7 @@ class Annotater(object):
                 lines = stream.readlines()
                 self._text = "".join(lines)
                 if fileid == self.fileid:
-                    # FIXME: Right hand side parents
-                    self._annotated = reannotate([self._annotated],
-                        lines, revid)
+                    self._annotated = self.check_file(lines, revid, [self._annotated])
                 return
             stream.write(apply_txdelta_window(self._text, window))
         return apply_window
