@@ -73,6 +73,7 @@ from bzrlib.versionedfile import (
     FulltextContentFactory,
     )
 
+from bzrlib.plugins.svn import changes
 from bzrlib.plugins.svn.errors import (
     AbsentPath,
     InvalidFileName,
@@ -591,8 +592,8 @@ class FileRevisionBuildEditor(FileBuildEditor):
             actual_checksum = md5_strings(self.base_chunks)
             if base_checksum != actual_checksum:
                 raise TextChecksumMismatch(base_checksum, actual_checksum,
-                    self.editor.revmeta.branch_path,
-                    self.editor.revmeta.revnum)
+                    self.editor.revmeta.metarev.branch_path,
+                    self.editor.revmeta.metarev.revnum)
         self.chunks = []
         return apply_txdelta_handler_chunks(self.base_chunks, self.chunks)
 
@@ -614,8 +615,8 @@ class FileRevisionBuildEditor(FileBuildEditor):
             actual_checksum = md5sum.hexdigest()
             if checksum != actual_checksum:
                 raise TextChecksumMismatch(checksum, actual_checksum,
-                        self.editor.revmeta.branch_path,
-                        self.editor.revmeta.revnum)
+                        self.editor.revmeta.metarev.branch_path,
+                        self.editor.revmeta.metarev.revnum)
 
         if self.is_special is not None:
             self.is_symlink = (self.is_special and chunks_start_with_link(chunks))
@@ -666,6 +667,16 @@ class FileRevisionBuildEditor(FileBuildEditor):
             self.bzr_base_path, self.path, self.file_id, ie)
 
 
+def ensure_inventories_in_repo(repo, trees):
+    real_inv_vf = repo.inventories.without_fallbacks()
+    for t in trees:
+        revid = t.get_revision_id()
+        if revid == NULL_REVISION:
+            continue
+        if not real_inv_vf.get_parent_map([(revid, )]):
+            repo.add_inventory(revid, t.inventory, t.get_parent_ids())
+
+
 class RevisionBuildEditor(DeltaBuildEditor):
     """Implementation of the Subversion commit editor interface that builds a
     Bazaar revision.
@@ -730,8 +741,9 @@ class RevisionBuildEditor(DeltaBuildEditor):
         if path == u"":
             if self.lhs_parent_revmeta is not None:
                 try:
-                    (action, copyfrom_path, copyfrom_revnum, kind) = self.revmeta.metarev.paths.get(
-                        self.revmeta.metarev.branch_path)
+                    (action, copyfrom_path, copyfrom_revnum, kind) = (
+                        self.revmeta.metarev.paths.get(
+                        self.revmeta.metarev.branch_path))
                 except TypeError:
                     copyfrom_path = None
                 if copyfrom_path is None:
@@ -762,8 +774,10 @@ class RevisionBuildEditor(DeltaBuildEditor):
         """
         # Find the ancestor of self.revmeta with revnum revnum
         last_revmeta = None
-        for revmeta, mapping in self.source._iter_reverse_revmeta_mapping_history(
+        for revmeta, hidden, mapping in self.source._revmeta_provider._iter_reverse_revmeta_mapping_history(
             self.revmeta.branch_path, self.revmeta.metarev.revnum, revnum, self.mapping):
+            if hidden:
+                continue
             last_revmeta = revmeta
         assert last_revmeta is not None and last_revmeta.metarev.revnum == revnum
         revid = last_revmeta.get_revision_id(mapping)
@@ -1046,19 +1060,21 @@ class FileTreeDeltaBuildEditor(FileBuildEditor):
             entry_kind = 'symlink'
         else:
             entry_kind = 'file'
-        if self.change_kind == 'add':
-            if (self.copyfrom_path is not None and
-                self._get_map_id(self.path) is not None):
-                self.editor.delta.renamed.append((self.copyfrom_path,
-                    self.path, self.editor._get_new_file_id(self.path),
-                    entry_kind, text_changed, metadata_changed))
+        fileid = self.editor._get_new_file_id(self.path)
+        if self.editor._should_include(fileid, self.path):
+            if self.change_kind == 'add':
+                if (self.copyfrom_path is not None and
+                    self._get_map_id(self.path) is not None):
+                    self.editor.delta.renamed.append((self.copyfrom_path,
+                        self.path, fileid,
+                        entry_kind, text_changed, metadata_changed))
+                else:
+                    self.editor.delta.added.append((self.path,
+                        fileid, entry_kind))
             else:
-                self.editor.delta.added.append((self.path,
-                    self.editor._get_new_file_id(self.path), entry_kind))
-        else:
-            self.editor.delta.modified.append((self.path,
-                self.editor._get_new_file_id(self.path), entry_kind,
-                text_changed, metadata_changed))
+                self.editor.delta.modified.append((self.path,
+                    fileid, entry_kind,
+                    text_changed, metadata_changed))
 
     def _apply_textdelta(self, base_checksum=None):
         self.base_checksum = None
@@ -1084,17 +1100,21 @@ class DirectoryTreeDeltaBuildEditor(DirectoryBuildEditor):
 
     def _delete_entry(self, path, revnum):
         # FIXME: old kind
-        self.editor.delta.removed.append((path,
-            self.editor._get_bzr_base_file_id(path), 'unknown'))
+        fileid = self.editor._get_bzr_base_file_id(path)
+        if self.editor._should_include(fileid, path):
+            self.editor.delta.removed.append((path,
+                fileid, 'unknown'))
 
     def _add_directory(self, path, copyfrom_path=None, copyfrom_revnum=-1):
-        if (copyfrom_path is not None and
-            self.editor._was_renamed(path) is not None):
-            self.editor.delta.renamed.append((copyfrom_path, path,
-                self.editor._get_new_file_id(path), 'directory', False, False))
-        else:
-            self.editor.delta.added.append((path,
-                self.editor._get_new_file_id(path), 'directory'))
+        fileid = self.editor._get_new_file_id(path)
+        if self.editor._should_include(fileid, path):
+            if (copyfrom_path is not None and
+                self.editor._was_renamed(path) is not None):
+                self.editor.delta.renamed.append((copyfrom_path, path,
+                    fileid, 'directory', False, False))
+            else:
+                self.editor.delta.added.append((path,
+                    fileid, 'directory'))
         return DirectoryTreeDeltaBuildEditor(self.editor, path)
 
 
@@ -1103,15 +1123,33 @@ class TreeDeltaBuildEditor(DeltaBuildEditor):
     Bazaar TreeDelta.
     """
 
-    def __init__(self, revmeta, mapping, newfileidmap, oldfileidmap):
+    def __init__(self, revmeta, mapping, newfileidmap, oldfileidmap,
+                 specific_fileids=None):
         super(TreeDeltaBuildEditor, self).__init__(revmeta, mapping)
+        self.specific_fileids = specific_fileids
         self._parent_idmap = oldfileidmap
         self._idmap = newfileidmap
+        if self.specific_fileids is not None:
+            self.specific_paths = [self._idmap.reverse_lookup(self.mapping, fileid) for fileid in self.specific_fileids]
         self.delta = delta.TreeDelta()
         self.delta.unversioned = []
         # To make sure we fall over if anybody tries to use it:
         self.delta.unchanged = None
 
+    def _should_include(self, file_id, path):
+        if self.specific_fileids is None:
+            return True
+        if file_id in self.specific_fileids:
+            return True
+        # path is under one of our specific_fileids
+        if changes.under_prefixes(path, self.specific_paths):
+            return True
+        # paht is a parent of one of our specific_fileids
+        for p in self.specific_paths:
+            if changes.path_is_child(path, p):
+                return True
+        return False
+
     def _open_root(self, base_revnum):
         return DirectoryTreeDeltaBuildEditor(self, u"")
 
@@ -1125,6 +1163,13 @@ class TreeDeltaBuildEditor(DeltaBuildEditor):
     def _get_new_file_id(self, path):
         return self._idmap.lookup(self.mapping, path)[0]
 
+    def sort(self):
+        self.delta.removed.sort()
+        self.delta.added.sort()
+        self.delta.renamed.sort()
+        self.delta.missing.sort()
+        self.delta.modified.sort()
+
 
 @convert_svn_error
 def report_inventory_contents(reporter, revnum, start_empty):
@@ -1180,7 +1225,7 @@ class FetchRevisionFinder(object):
         return [k for k in revmetas if k in map and map[k] not in present_revids]
 
     def find_iter_revisions(self, iter, master_mapping, needs_manual_check,
-                            heads=None, pb=None):
+                            heads=None, pb=None, exclude_non_mainline=None):
         """Find revisions to fetch based on an iterator over available revmetas.
 
         :param iter: Iterator over RevisionMetadata objects
@@ -1216,7 +1261,8 @@ class FetchRevisionFinder(object):
                 if (lhs_parent_revmeta is not None and
                     needs_manual_check(lhs_parent_revmeta)):
                     self.needed.extend(self.find_mainline(
-                        lhs_parent_revmeta.metarev.get_foreign_revid(), lhsm))
+                        lhs_parent_revmeta.metarev.get_foreign_revid(), lhsm,
+                        exclude_non_mainline=exclude_non_mainline))
                 if lhsm != master_mapping or heads is not None:
                     needed_mappings[lhs_parent_revmeta].add(lhsm)
                 if not revmeta.is_hidden(m):
@@ -1240,7 +1286,8 @@ class FetchRevisionFinder(object):
         finally:
             pb.finished()
 
-    def find_mainline(self, foreign_revid, mapping, find_ghosts=False):
+    def find_mainline(self, foreign_revid, mapping, find_ghosts=False,
+                      exclude_non_mainline=None):
         if (foreign_revid, mapping) in self.checked:
             return []
         revmetas = deque()
@@ -1250,8 +1297,10 @@ class FetchRevisionFinder(object):
         needs_checking = []
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            for revmeta, mapping in self.source._revmeta_provider._iter_reverse_revmeta_mapping_history(
+            for revmeta, hidden, mapping in self.source._revmeta_provider._iter_reverse_revmeta_mapping_history(
                 branch_path, revnum, to_revnum=0, mapping=mapping):
+                if hidden:
+                    continue
                 if pb:
                     pb.update("determining revisions to fetch",
                               revnum-revmeta.metarev.revnum, revnum)
@@ -1276,7 +1325,13 @@ class FetchRevisionFinder(object):
         for r in self.check_revmetas(needs_checking):
             revmetas.appendleft(r)
         # Determine if there are any RHS parents to fetch
-        self.extra.extend(self.find_rhs_parents(revmetas))
+        if exclude_non_mainline is None:
+            # FIXME JRV 2011-10-20: Fetching non-mainline revisions
+            # doesn't work yet - they end up on the branch mainline
+            # and cause diverged branch errors.
+            exclude_non_mainline = isinstance(self.target, SvnRepository)
+        if not exclude_non_mainline:
+            self.extra.extend(self.find_rhs_parents(revmetas))
         return revmetas
 
     def find_rhs_parents(self, revmetas):
@@ -1292,7 +1347,8 @@ class FetchRevisionFinder(object):
                     if self.source.has_foreign_revision(foreign_revid):
                         yield (foreign_revid, rhs_mapping)
 
-    def find_until(self, foreign_revid, mapping, find_ghosts=False):
+    def find_until(self, foreign_revid, mapping, find_ghosts=False,
+                   exclude_non_mainline=None):
         """Find all missing revisions until revision_id
 
         :param revision_id: Stop revision
@@ -1303,7 +1359,8 @@ class FetchRevisionFinder(object):
         while len(self.extra) > 0:
             foreign_revid, mapping = self.extra.pop()
             self.needed.extend(self.find_mainline(foreign_revid, mapping,
-                find_ghosts=find_ghosts))
+                find_ghosts=find_ghosts,
+                exclude_non_mainline=exclude_non_mainline))
 
 
 class InterFromSvnRepository(InterRepository):
@@ -1377,6 +1434,7 @@ class InterFromSvnRepository(InterRepository):
         try:
             bzr_parent_trees = self._get_parent_trees(revmeta, mapping,
                 lhs_parent_revmeta)
+            ensure_inventories_in_repo(self.target, bzr_parent_trees)
             svn_base_tree = bzr_parent_trees[0]
             return RevisionBuildEditor(self.source, self.target, revid,
                 bzr_parent_trees, svn_base_tree,
@@ -1513,7 +1571,8 @@ class InterFromSvnRepository(InterRepository):
         return FetchRevisionFinder(self.source, self.target, target_is_empty)
 
     def _get_needed(self, revision_id=None, fetch_spec=None, project=None,
-                    target_is_empty=False, find_ghosts=False):
+                    target_is_empty=False, find_ghosts=False,
+                    exclude_non_mainline=None):
         """Find the set of revisions that is missing.
 
         :note: revision_id and fetch_spec are mutually exclusive
@@ -1532,7 +1591,8 @@ class InterFromSvnRepository(InterRepository):
             foreign_revid, mapping = self.source.lookup_bzr_revision_id(
                 revision_id, project=project)
             revisionfinder.find_until(foreign_revid, mapping,
-                                      find_ghosts=find_ghosts)
+                                      find_ghosts=find_ghosts,
+                                      exclude_non_mainline=exclude_non_mainline)
         elif fetch_spec is not None:
             recipe = fetch_spec.get_recipe()
             if recipe[0] in ("search", "proxy-search"):
@@ -1543,14 +1603,15 @@ class InterFromSvnRepository(InterRepository):
                 foreign_revid, mapping = self.source.lookup_bzr_revision_id(
                     head, project=project)
                 revisionfinder.find_until(foreign_revid, mapping,
-                    find_ghosts=find_ghosts)
+                    find_ghosts=find_ghosts,
+                    exclude_non_mainline=exclude_non_mainline)
         else:
             revisionfinder.find_all(self.source.get_mapping())
         return revisionfinder.get_missing()
 
     def fetch(self, revision_id=None, pb=None, find_ghosts=False,
               needed=None, mapping=None, project=None, fetch_spec=None,
-              target_is_empty=False):
+              target_is_empty=False, exclude_non_mainline=None):
         """Fetch revisions. """
         if revision_id == NULL_REVISION:
             return
@@ -1567,7 +1628,8 @@ class InterFromSvnRepository(InterRepository):
             if needed is None:
                 needed = self._get_needed(target_is_empty=target_is_empty,
                     revision_id=revision_id, fetch_spec=fetch_spec,
-                    find_ghosts=find_ghosts, project=project)
+                    find_ghosts=find_ghosts, project=project,
+                    exclude_non_mainline=exclude_non_mainline)
 
             if len(needed) == 0:
                 # Nothing to fetch
