@@ -13,17 +13,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-try:
-    from functools import partial
-except ImportError:
-    from bzrlib.plugins.svn.pycompat import partial
+from functools import partial
 
-try:
-    all
-except NameError:
-    from bzrlib.plugins.svn.pycompat import all
-
-from subvertpy import NODE_DIR
+import subvertpy
 
 from bzrlib import urlutils
 from bzrlib.plugins.svn import errors as svn_errors
@@ -86,7 +78,10 @@ class TrunkLayout(RepositoryLayout):
         :param project: Optional name of the project the branch is for. Can include slashes.
         :return: Path of the branch.
         """
-        return urlutils.join(project, "branches", name).strip("/")
+        if name is None:
+            return urlutils.join(project, "trunk").strip("/")
+        else:
+            return urlutils.join(project, "branches", name).strip("/")
 
     def parse(self, path):
         """Parse a path.
@@ -94,7 +89,7 @@ class TrunkLayout(RepositoryLayout):
         :return: Tuple with type ('tag', 'branch'), project name, branch path and path
             inside the branch
         """
-        assert isinstance(path, str)
+        assert type(path) is str
         path = path.strip("/")
         parts = path.split("/")
         for i, p in enumerate(parts):
@@ -120,21 +115,22 @@ class TrunkLayout(RepositoryLayout):
             return path
         return urlutils.join(project, path)
 
-    def get_branches(self, repository, revnum, project=None, pb=None):
+    def get_branches(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
-        :return: Iterator over tuples with (project, branch path, has_props)
+        :return: Iterator over tuples with (project, branch path, has_props, revnum)
         """
         return get_root_paths(repository,
              [self._add_project(x, project) for x in "branches/*", "trunk"],
              revnum, self.is_branch, project)
 
-    def get_tags(self, repository, revnum, project=None, pb=None):
+    def get_tags(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
-        :return: Iterator over tuples with (project, branch path)
+        :return: Iterator over tuples with (project, branch path, has_props, revnum)
         """
-        return get_root_paths(repository, [self._add_project("tags/*", project)], revnum, self.is_tag, project)
+        return get_root_paths(repository, [self._add_project("tags/*", project)],
+                revnum, self.is_tag, project)
 
     def __repr__(self):
         if self.level is None:
@@ -173,7 +169,8 @@ class RootLayout(RepositoryLayout):
         :param project: Optional name of the project the tag is for. Can include slashes.
         :return: Path of the tag."
         """
-        return None
+        raise svn_errors.NoLayoutTagSetSupport(self,
+            "the root layout does not support tags")
 
     def get_tag_name(self, path, project=""):
         """Determine the tag name from a tag path.
@@ -182,15 +179,6 @@ class RootLayout(RepositoryLayout):
         """
         raise AssertionError("should never be reached, there can't be any tag paths in this layout")
 
-    def push_merged_revisions(self, project=""):
-        """Determine whether or not right hand side (merged) revisions should be pushed.
-
-        Defaults to False.
-
-        :param project: Name of the project.
-        """
-        return False
-
     def get_branch_path(self, name, project=""):
         """Return the path at which the branch with specified name should be found.
 
@@ -198,6 +186,8 @@ class RootLayout(RepositoryLayout):
         :param project: Optional name of the project the branch is for. Can include slashes.
         :return: Path of the branch.
         """
+        if name is not None or project:
+            raise svn_errors.NoCustomBranchPaths(self)
         return ""
 
     def parse(self, path):
@@ -214,17 +204,17 @@ class RootLayout(RepositoryLayout):
     def is_tag_path(self, tp, project=None):
         return False
 
-    def get_branches(self, repository, revnum, project=None, pb=None):
+    def get_branches(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
         """
-        return [("", "", "trunk", None)]
+        return [("", "", "trunk", None, revnum)]
 
-    def get_tags(self, repository, revnum, project=None, pb=None):
+    def get_tags(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
-        :return: Iterator over tuples with (project, branch path)
+        :return: Iterator over tuples with (project, branch path, branch name, has_props)
         """
         return []
 
@@ -251,7 +241,7 @@ class CustomLayout(RepositoryLayout):
         :param project: Optional name of the project the tag is for. Can include slashes.
         :return: Path of the tag.
         """
-        return None
+        raise svn_errors.NoLayoutTagSetSupport(self)
 
     def get_tag_name(self, path, project=""):
         """Determine the tag name from a tag path.
@@ -260,24 +250,6 @@ class CustomLayout(RepositoryLayout):
         """
         return None
 
-    def push_merged_revisions(self, project=""):
-        """Determine whether or not right hand side (merged) revisions should be pushed.
-
-        Defaults to False.
-
-        :param project: Name of the project.
-        """
-        return False
-
-    def get_branch_path(self, name, project=""):
-        """Return the path at which the branch with specified name should be found.
-
-        :param name: Name of the branch.
-        :param project: Optional name of the project the branch is for. Can include slashes.
-        :return: Path of the branch.
-        """
-        raise svn_errors.NoCustomBranchPaths(self)
-
     def parse(self, path):
         """Parse a path.
 
@@ -295,19 +267,34 @@ class CustomLayout(RepositoryLayout):
 
         raise svn_errors.NotSvnBranchPath(path)
 
-    def get_branches(self, repository, revnum, project=None, pb=None):
+    def _get_paths(self, entries, project, repository, revnum):
+        ret = []
+        for b in entries:
+            try:
+                r = repository.transport.get_dir(b, revnum)[1]
+            except subvertpy.SubversionException, (msg, num):
+                if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
+                           subvertpy.ERR_FS_NOT_FOUND,
+                           subvertpy.ERR_RA_DAV_PATH_NOT_FOUND,
+                           subvertpy.ERR_RA_DAV_FORBIDDEN):
+                    continue
+                raise
+            ret.append((project, b, b.split("/")[-1], None, r))
+        return ret
+
+    def get_branches(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
         """
-        return [(project, b, b.split("/")[-1], None) for b in self.branches if repository.transport.check_path(b, revnum) == NODE_DIR]
+        return self._get_paths(self.branches, project, repository, revnum)
 
-    def get_tags(self, repository, revnum, project=None, pb=None):
+    def get_tags(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
         """
-        return [(project, t, t.split("/")[-1], None) for t in self.tags if repository.transport.check_path(t, revnum) == NODE_DIR]
+        return self._get_paths(self.tags, project, repository, revnum)
 
     def __repr__(self):
         return "%s(%r,%r)" % (self.__class__.__name__, self.branches, self.tags)
@@ -330,7 +317,7 @@ class WildcardLayout(RepositoryLayout):
     def __init__(self, branches=[], tags=[]):
         self.branches = [b.strip("/") for b in branches]
         self.tags = [t.strip("/") for t in tags]
-        assert all([isinstance(b, str) for b in self.branches + self.tags])
+        assert all([type(b) is str for b in self.branches + self.tags])
 
     def supports_tags(self):
         return (self.tags != [])
@@ -342,8 +329,18 @@ class WildcardLayout(RepositoryLayout):
         :param project: Optional name of the project the tag is for. Can include slashes.
         :return: Path of the tag."
         """
-        # FIXME
-        return None
+        if project:
+            raise svn_errors.NoLayoutTagSetSupport(self)
+        if len(self.tags) == 0:
+            raise svn_errors.NoLayoutTagSetSupport(self,
+                "no tag paths set")
+        if self.tags[0].count("*") == 0:
+            raise svn_errors.NoLayoutTagSetSupport(self,
+                "no asterisk in tag path")
+        if self.tags[0].count("*") > 1:
+            raise svn_errors.NoLayoutTagSetSupport(self,
+                "can only handle a single asterisk in tag path")
+        return self.tags[0].replace("*", name)
 
     def get_tag_name(self, path, project=""):
         """Determine the tag name from a tag path.
@@ -358,27 +355,9 @@ class WildcardLayout(RepositoryLayout):
                 return path.split("/")[-1]
         return None
 
-    def push_merged_revisions(self, project=""):
-        """Determine whether or not right hand side (merged) revisions should be pushed.
-
-        Defaults to False.
-
-        :param project: Name of the project.
-        """
-        return False
-
-    def get_branch_path(self, name, project=""):
-        """Return the path at which the branch with specified name should be found.
-
-        :param name: Name of the branch.
-        :param project: Optional name of the project the branch is for. Can include slashes.
-        :return: Path of the branch.
-        """
-        raise svn_errors.NoCustomBranchPaths(self)
-
     def is_branch(self, path, project=None):
         for bp in self.branches:
-           if wildcard_matches(path, bp):
+            if wildcard_matches(path, bp):
                 return True
         return False
 
@@ -405,7 +384,7 @@ class WildcardLayout(RepositoryLayout):
 
         raise svn_errors.NotSvnBranchPath(path)
 
-    def get_branches(self, repository, revnum, project=None, pb=None):
+    def get_branches(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
@@ -413,7 +392,7 @@ class WildcardLayout(RepositoryLayout):
         return get_root_paths(repository, self.branches,
              revnum, self.is_branch, project)
 
-    def get_tags(self, repository, revnum, project=None, pb=None):
+    def get_tags(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
@@ -448,15 +427,6 @@ class InverseTrunkLayout(RepositoryLayout):
         """
         return urlutils.basename(path).strip("/")
 
-    def push_merged_revisions(self, project=""):
-        """Determine whether or not right hand side (merged) revisions should be pushed.
-
-        Defaults to False.
-
-        :param project: Name of the project.
-        """
-        return False
-
     def get_branch_path(self, name, project=""):
         """Return the path at which the branch with specified name should be found.
 
@@ -472,7 +442,7 @@ class InverseTrunkLayout(RepositoryLayout):
         :return: Tuple with type ('tag', 'branch'), project name, branch path and path
             inside the branch
         """
-        assert isinstance(path, str)
+        assert type(path) is str
         path = path.strip("/")
         parts = path.split("/")
         if len(parts) == 0:
@@ -480,7 +450,10 @@ class InverseTrunkLayout(RepositoryLayout):
         if parts[0] == "trunk":
             if len(parts) < (self.level + 1):
                 raise svn_errors.NotSvnBranchPath(path)
-            return ("branch", "/".join(parts[1:self.level+2]), "/".join(parts[:self.level+1]), "/".join(parts[self.level+1:]))
+            return ("branch",
+                    "/".join(parts[1:self.level+2]),
+                    "/".join(parts[:self.level+1]),
+                    "/".join(parts[self.level+1:]))
         elif parts[0] in ("branches", "tags"):
             if len(parts) < (self.level + 2):
                 raise svn_errors.NotSvnBranchPath(path)
@@ -488,7 +461,10 @@ class InverseTrunkLayout(RepositoryLayout):
                 t = "branch"
             else:
                 t = "tag"
-            return (t, "/".join(parts[1:self.level+1]), "/".join(parts[:self.level+2]), "/".join(parts[self.level+2:]))
+            return (t,
+                    "/".join(parts[1:self.level+1]),
+                    "/".join(parts[:self.level+2]),
+                    "/".join(parts[self.level+2:]))
         raise svn_errors.NotSvnBranchPath(path)
 
     def _add_project(self, path, project=None):
@@ -499,8 +475,8 @@ class InverseTrunkLayout(RepositoryLayout):
         else:
             return urlutils.join(urlutils.join(path, project), "*")
 
-    def get_branches(self, repository, revnum, project=None, pb=None):
-        """Retrieve a list of paths that refer to branches in a specific revision.
+    def get_branches(self, repository, revnum, project=None):
+        """Return a list of paths that refer to branches in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
         """
@@ -508,12 +484,14 @@ class InverseTrunkLayout(RepositoryLayout):
              [self._add_project(x, project) for x in "branches", "trunk"],
              revnum, self.is_branch, project)
 
-    def get_tags(self, repository, revnum, project=None, pb=None):
+    def get_tags(self, repository, revnum, project=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
 
         :return: Iterator over tuples with (project, branch path)
         """
-        return get_root_paths(repository, [self._add_project("tags", project)], revnum, self.is_tag, project)
+        return get_root_paths(repository,
+                [self._add_project("tags", project)], revnum, self.is_tag,
+                project)
 
     def __repr__(self):
         return "%s(%d)" % (self.__class__.__name__, self.level)
