@@ -34,6 +34,7 @@ from bzrlib.errors import (
     BzrError,
     DivergedBranches,
     NoSuchRevision,
+    NotWriteLocked,
     )
 from bzrlib.repository import (
     InterRepository,
@@ -73,6 +74,7 @@ from bzrlib.plugins.svn.transport import (
     url_join_unescaped_path,
     )
 
+ERR_FS_ROOT_DIR = getattr(subvertpy, "ERR_FS_ROOT_DIR", 160021)
 
 def create_branch_container(transport, prefix, already_present):
     """Create a branch prefix.
@@ -127,6 +129,8 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
     :param root_action: Action to take on the tree root
     :return: Revision id of newly created revision.
     """
+    if target_repo._lock_mode != 'w':
+        raise NotWriteLocked(target_repo)
     assert rev.revision_id in (None, revision_id)
     old_tree = source_repo.revision_tree(revision_id)
     if rev.parent_ids:
@@ -159,7 +163,9 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
                                root_action, base_tree,
                                push_metadata=push_metadata,
                                graph=graph, opt_signature=opt_signature,
-                               testament=testament)
+                               testament=testament,
+                               _rich_root_bump=not source_repo.supports_rich_root())
+    target_repo.start_write_group()
     try:
         builder.will_record_deletes()
         iter_changes = old_tree.iter_changes(base_tree)
@@ -174,6 +180,7 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
     try:
         revid = builder.commit(rev.message)
     except SubversionException, (msg, num):
+        builder.abort()
         if num == ERR_FS_TXN_OUT_OF_DATE:
             raise DivergedBranches(source_repo, target_repo)
         raise
@@ -253,10 +260,11 @@ class InterToSvnRepository(InterRepository):
         if len(existing_bp_parts) < len(bp_parts):
             # Branch doesn't exist yet
             return ("create", )
-        (revmeta, mapping) = self.target._revmeta_provider._iter_reverse_revmeta_mapping_history(
+        (revmeta, hidden, mapping) = self.target._revmeta_provider._iter_reverse_revmeta_mapping_history(
             path, self.target.get_latest_revnum(), to_revnum=0,
             mapping=self.target.get_mapping()).next()
-        if parent_ids == [] or parent_ids == [NULL_REVISION]:
+        assert not hidden
+        if tuple(parent_ids) == () or tuple(parent_ids) == (NULL_REVISION,):
             return ("replace", revmeta.metarev.revnum)
         else:
             if revmeta.get_revision_id(mapping) != parent_ids[0]:
@@ -265,7 +273,7 @@ class InterToSvnRepository(InterRepository):
                         urlutils.join(self.target.base, path))
                 return ("replace", revmeta.metarev.revnum)
             else:
-                return ("open", )
+                return ("open", revmeta.metarev.revnum)
 
     def _otherline_missing_revisions(self, graph, stop_revision, project, overwrite):
         """Find the revisions missing on the mainline.
@@ -295,25 +303,26 @@ class InterToSvnRepository(InterRepository):
             for revid in graph.iter_lefthand_ancestry(stop_revision,
                     (NULL_REVISION, None)):
                 if revid == last_revid:
-                    todo.reverse()
                     break
                 todo.append(revid)
             else:
-                url = urlutils.join(self.target.base, target_branch)
-                raise AppendRevisionsOnlyViolation(url)
-            return todo, ("open", )
+                if last_revid != NULL_REVISION:
+                    url = urlutils.join(self.target.base, target_branch)
+                    raise AppendRevisionsOnlyViolation(url)
+            todo.reverse()
+            return todo, ("open", last_foreign_revid[2])
         else:
-            for revid in graph.iter_lefthand_ancestry(stop_revision, (NULL_REVISION, None)):
+            for revid in graph.iter_lefthand_ancestry(stop_revision,
+                    (NULL_REVISION, None)):
                 if self._target_has_revision(revid, project=project):
-                    todo.reverse()
                     break
                 todo.append(revid)
+            todo.reverse()
             return todo, ("replace", last_foreign_revid[2])
 
-    def push_branch(self, target_branch_path, target_config,
-            (last_revmeta, mapping), stop_revision, layout, project,
+    def push_branch(self, target_branch_path, target_config, old_last_revid,
+            old_last_foreign_revid, mapping, stop_revision, layout, project,
             overwrite, push_metadata, push_merged):
-        old_last_revid = last_revmeta.get_revision_id(mapping)
         graph = self.get_graph()
         if not overwrite:
             heads = graph.heads([old_last_revid, stop_revision])
@@ -326,7 +335,7 @@ class InterToSvnRepository(InterRepository):
                         self.target, target_branch_path, old_last_revid)
         append_revisions_only = target_config.get_append_revisions_only(
             not overwrite)
-        return self.push_todo(old_last_revid, last_revmeta.metarev.get_foreign_revid(),
+        return self.push_todo(old_last_revid, old_last_foreign_revid,
             mapping, stop_revision, layout, project,
             target_branch_path, target_config, push_merged=push_merged,
             overwrite=overwrite, push_metadata=push_metadata,
@@ -390,20 +399,21 @@ class InterToSvnRepository(InterRepository):
                     push_metadata=push_metadata, push_merged=push_merged,
                     project=project, layout=layout)
                 revid_map[rev.revision_id] = (last_revid, last_foreign_info)
-                root_action = ("open", )
+                root_action = ("open", last_foreign_info[0][2])
             return revid_map
         finally:
             pb.finished()
 
     def push_revision_inclusive(self, target_path, target_config, rev,
             push_merged, layout, project, root_action, push_metadata,
-            base_foreign_info):
+            base_foreign_info, exclude=None):
         """Push a revision including ancestors.
 
         :return: Tuple with pushed revision id and foreign revision id
         """
         if push_merged and len(rev.parent_ids) > 1:
-            self.push_ancestors(layout, project, rev.parent_ids)
+            self.push_ancestors(layout, project, rev.parent_ids,
+                    exclude=[target_path])
         return self.push_single_revision(target_path, target_config, rev,
             push_metadata=push_metadata, root_action=root_action,
             base_foreign_info=base_foreign_info)
@@ -447,7 +457,8 @@ class InterToSvnRepository(InterRepository):
         return BranchConfig(urlutils.join(self.target.base, branch_path),
                 self.target.uuid)
 
-    def push_ancestors(self, layout, project, parent_revids):
+    def push_ancestors(self, layout, project, parent_revids, lossy=False,
+                       exclude=None):
         """Push the ancestors of a revision.
 
         :param layout: Subversion layout
@@ -460,7 +471,8 @@ class InterToSvnRepository(InterRepository):
         graph = self.get_graph()
         for parent_revid in missing_rhs_parents:
             # Push merged revisions
-            ancestors = graph.find_unique_ancestors(parent_revid, [parent_revids[0]])
+            ancestors = graph.find_unique_ancestors(parent_revid,
+                [parent_revids[0]])
             unique_ancestors.update(ancestors)
         for x in self.get_graph().iter_topo_order(unique_ancestors):
             if self._target_has_revision(x, project):
@@ -480,7 +492,7 @@ class InterToSvnRepository(InterRepository):
                 target_project = None
             else:
                 (_, target_project, _, _) = layout.parse(base_foreign_revid[1])
-            bp = determine_branch_path(rev, layout, target_project)
+            bp = determine_branch_path(rev, layout, target_project, exclude)
             target_config = self._get_branch_config(bp)
             push_merged = (layout.push_merged_revisions(target_project) and
                 target_config.get_push_merged_revisions())
@@ -489,7 +501,7 @@ class InterToSvnRepository(InterRepository):
                 append_revisions_only=target_config.get_append_revisions_only(False),
                 create_prefix=True)
             self.push_revision_inclusive(bp, target_config, rev,
-                push_metadata=True, push_merged=push_merged,
+                push_metadata=not lossy, push_merged=push_merged,
                 layout=layout, project=target_project,
                 root_action=root_action,
                 base_foreign_info=(base_foreign_revid, base_mapping))
@@ -559,60 +571,89 @@ class InterToSvnRepository(InterRepository):
             self._graph = self.source.get_graph(self.target)
         return self._graph
 
-    def copy_content(self, revision_id=None, pb=None, project=None, mapping=None):
+    def copy_content(self, revision_id=None, pb=None, project=None,
+            mapping=None, limit=None, lossy=False, exclude_non_mainline=None):
         """See InterRepository.copy_content."""
         self.source.lock_read()
         try:
-            assert revision_id is not None, "fetching all revisions not supported"
-            graph = self.get_graph()
-            todo = []
-            # Go back over the LHS parent until we reach a revid we know
-            for revid in graph.iter_lefthand_ancestry(revision_id,
-                    (NULL_REVISION, None)):
-                if self._target_has_revision(revid):
-                    break
-                todo.append(revid)
-            else:
-                return
-            mutter("pushing %r into svn", todo)
-            base_foreign_info = None
-            layout = self.target.get_layout()
-            for rev in self.source.get_revisions(todo):
-                if pb is not None:
-                    pb.update("pushing revisions", todo.index(rev.revision_id),
-                        len(todo))
-                mutter('pushing %r', rev.revision_id)
-
-                if base_foreign_info is None:
-                    if rev.parent_ids:
-                        base_revid = rev.parent_ids[0]
-                    else:
-                        base_revid = NULL_REVISION
-                    base_foreign_info  = self._get_foreign_revision_info(
-                        base_revid)
-
-                (base_foreign_revid, base_mapping) = base_foreign_info
-                if base_foreign_revid is None:
-                    target_project = None
+            self.target.lock_write()
+            try:
+                graph = self.get_graph()
+                if revision_id is not None:
+                    heads = [revision_id]
                 else:
-                    (_, target_project, _, _) = layout.parse(base_foreign_revid[1])
-                bp = determine_branch_path(rev, layout, target_project)
-                target_config = self._get_branch_config(bp)
-                push_merged = (layout.push_merged_revisions(target_project) and
-                    target_config.get_push_merged_revisions())
-                root_action = self._get_root_action(bp, rev.parent_ids, overwrite=False,
-                    append_revisions_only=target_config.get_append_revisions_only(True),
-                    create_prefix=True)
-                (pushed_revid, base_foreign_info) = self.push_revision_inclusive(
-                    bp, target_config, rev, push_metadata=True,
-                    push_merged=push_merged, root_action=root_action,
-                    layout=layout, project=target_project,
-                    base_foreign_info=base_foreign_info)
+                    heads = graph.heads(self.source.all_revision_ids())
+                    exclude_non_mainline = False
+                todo = []
+                # Go back over the LHS parent until we reach a revid we know
+                for head in heads:
+                    for revid in graph.iter_lefthand_ancestry(head,
+                            (NULL_REVISION, None)):
+                        if self._target_has_revision(revid):
+                            break
+                        todo.append(revid)
+                todo.reverse()
+                if limit is not None:
+                    # FIXME: This only considers mainline revisions.
+                    # Properly keeping track of how many revisions have been
+                    # pushed will be fairly complicated though, so for the
+                    # moment this is reasonable enough (and passes tests).
+                    todo = todo[:limit]
+                mutter("pushing %r into svn", todo)
+                base_foreign_info = None
+                layout = self.target.get_layout()
+                for rev in self.source.get_revisions(todo):
+                    if pb is not None:
+                        pb.update("pushing revisions",
+                            todo.index(rev.revision_id), len(todo))
+                    mutter('pushing %r', rev.revision_id)
+
+                    if base_foreign_info is None:
+                        if rev.parent_ids:
+                            base_revid = rev.parent_ids[0]
+                        else:
+                            base_revid = NULL_REVISION
+                        base_foreign_info  = self._get_foreign_revision_info(
+                            base_revid)
+
+                    (base_foreign_revid, base_mapping) = base_foreign_info
+                    if base_foreign_revid is None:
+                        target_project = None
+                    else:
+                        (_, target_project, _, _) = layout.parse(
+                            base_foreign_revid[1])
+                    bp = determine_branch_path(rev, layout, target_project)
+                    mutter("pushing revision include %r to %s",
+                            rev.revision_id, bp)
+                    target_config = self._get_branch_config(bp)
+                    can_push_merged = layout.push_merged_revisions(target_project)
+                    if exclude_non_mainline is None:
+                        push_merged = can_push_merged and (
+                            target_config.get_push_merged_revisions())
+                    else:
+                        push_merged = (not exclude_non_mainline)
+                    if push_merged and not can_push_merged:
+                        raise BzrError(
+                            "Unable to push merged revisions, layout "
+                            "does not provide branch path")
+                    root_action = self._get_root_action(bp, rev.parent_ids,
+                        overwrite=False,
+                        append_revisions_only=target_config.get_append_revisions_only(True),
+                        create_prefix=True)
+                    (pushed_revid,
+                            base_foreign_info) = self.push_revision_inclusive(
+                        bp, target_config, rev, push_metadata=not lossy,
+                        push_merged=push_merged, root_action=root_action,
+                        layout=layout, project=target_project,
+                        base_foreign_info=base_foreign_info)
+            finally:
+                self.source.unlock()
         finally:
-            self.source.unlock()
+            self.target.unlock()
 
     def fetch(self, revision_id=None, pb=None, find_ghosts=False,
-        fetch_spec=None, project=None, mapping=None, target_is_empty=False):
+        fetch_spec=None, project=None, mapping=None, target_is_empty=False,
+        limit=None, exclude_non_mainline=None):
         """Fetch revisions. """
         if fetch_spec is not None:
             recipe = fetch_spec.get_recipe()
@@ -622,10 +663,12 @@ class InterToSvnRepository(InterRepository):
                 raise AssertionError("Unknown search type %s" % recipe[0])
             for revid in heads:
                 self.copy_content(revision_id=revid, pb=pb, project=project,
-                    mapping=mapping)
+                    mapping=mapping, limit=limit,
+                    exclude_non_mainline=exclude_non_mainline)
         else:
             self.copy_content(revision_id=revision_id, pb=pb, project=project,
-                mapping=mapping)
+                    mapping=mapping, limit=limit,
+                    exclude_non_mainline=exclude_non_mainline)
 
     @staticmethod
     def is_compatible(source, target):
@@ -635,7 +678,7 @@ class InterToSvnRepository(InterRepository):
         return True
 
 
-def determine_branch_path(rev, layout, project=None):
+def determine_branch_path(rev, layout, project=None, exclude=None):
     """Create a sane branch path to use for a revision.
 
     :param rev: Revision object
@@ -643,12 +686,19 @@ def determine_branch_path(rev, layout, project=None):
     :param project: Optional project name, as used by the layout
     :return: Branch path string
     """
+    if exclude is None:
+        exclude = []
+    bp = None
     nick = (rev.properties.get('branch-nick') or "merged")
     nick = nick.encode("utf-8").replace("/","_")
-    if project is None:
-        return layout.get_branch_path(nick)
-    else:
-        return layout.get_branch_path(nick, project)
+    while True:
+        if project is None:
+            bp = layout.get_branch_path(nick)
+        else:
+            bp = layout.get_branch_path(nick, project)
+        if bp not in exclude:
+            return bp
+        nick += '-merged'
 
 
 def create_branch_with_hidden_commit(repository, branch_path, revid,
@@ -715,7 +765,12 @@ def create_branch_with_hidden_commit(repository, branch_path, revid,
         try:
             root = ci.open_root()
             if deletefirst:
-                root.delete_entry(urlutils.basename(branch_path))
+                try:
+                    root.delete_entry(urlutils.basename(branch_path))
+                except SubversionException, (msg, num):
+                    if num == subvertpy.ERR_FS_ROOT_DIR:
+                        raise ChangesRootLHSHistory()
+                    raise
             branch_dir = root.add_directory(
                 urlutils.basename(branch_path), from_url, from_revnum)
             for k, (ov, nv) in properties.diff(fileprops, old_fileprops).iteritems():
@@ -726,6 +781,7 @@ def create_branch_with_hidden_commit(repository, branch_path, revid,
             ci.abort()
             raise
         ci.close()
+        repository._cache_add_new_revision(foreign_revid[2], revid, None)
         return revid, (tuple(foreign_revid), mapping)
     finally:
         repository.transport.add_connection(conn)
