@@ -23,7 +23,9 @@ from bzrlib import (
     errors,
     osutils,
     trace,
+    urlutils,
     )
+from bzrlib.branch import InterBranch
 from bzrlib.controldir import (
     ControlDirFormat,
     ControlDir,
@@ -74,10 +76,14 @@ class UninitializableOnRemoteTransports(errors.UninitializableFormat):
 class SvnRemoteFormat(ControlDirFormat):
     """Format for the Subversion smart server."""
 
+    colocated_branches = True
     supports_workingtrees = False
     fixed_components = True
     _lock_class = TransportLock
 
+    def __eq__(self, other):
+        return isinstance(other, SvnRemoteFormat)
+
     @property
     def repository_format(self):
         from bzrlib.plugins.svn.repository import SvnRepositoryFormat
@@ -139,8 +145,11 @@ class SvnRemoteFormat(ControlDirFormat):
             transport.create_prefix()
 
         controldir = self.initialize_on_transport(transport)
-        repository = controldir.open_repository()
-        repository.lock_write()
+        if repo_format_name is not None:
+            repository = controldir.open_repository()
+            repository.lock_write()
+        else:
+            repository = None
         return (repository, controldir, None, CreateRepository(controldir))
 
     def initialize_on_transport(self, transport):
@@ -149,14 +158,22 @@ class SvnRemoteFormat(ControlDirFormat):
         lazy_check_versions()
         from bzrlib.transport.local import LocalTransport
         import os
+        import subvertpy
         from subvertpy import repos
+        # For subvertpy < 0.8.6
+        ERR_DIR_NOT_EMPTY = getattr(subvertpy, "ERR_DIR_NOT_EMPTY", 200011)
 
         if not isinstance(transport, LocalTransport):
             raise UninitializableOnRemoteTransports(self)
 
         local_path = transport.local_abspath(".").rstrip("/").encode(osutils._fs_enc)
         assert type(local_path) == str
-        repos.create(local_path)
+        try:
+            repos.create(local_path)
+        except subvertpy.SubversionException, (_, num):
+            if num == ERR_DIR_NOT_EMPTY:
+                raise errors.BzrError("Directory is not empty")
+            raise
         # All revision property changes
         revprop_hook = os.path.join(local_path, "hooks", "pre-revprop-change")
         open(revprop_hook, 'w').write("#!/bin/sh")
@@ -164,6 +181,18 @@ class SvnRemoteFormat(ControlDirFormat):
         from bzrlib.plugins.svn.transport import get_svn_ra_transport
         return self.open(get_svn_ra_transport(transport), _found=True)
 
+    def supports_transport(self, transport):
+        try:
+            url = transport.external_url()
+        except errors.InProcessTransport:
+            return False
+        if url.startswith("file:") or url.startswith("svn+"):
+            return True
+        if url.startswith("http:") or url.startswith("https:"):
+            # FIXME: Check that the server is running SVN
+            return False
+        return False
+
 
 class SvnRemoteAccess(ControlDir):
     """ControlDir implementation for Subversion connections.
@@ -191,12 +220,12 @@ class SvnRemoteAccess(ControlDir):
         self.transport = None
         self.root_transport = _transport
 
-        svn_url, readonly = bzr_to_svn_url(self.root_transport.base)
+        self.svn_url, readonly = bzr_to_svn_url(self.root_transport.base)
         self.svn_root_url = _transport.get_svn_repos_root()
         self.root_url = _transport.get_repos_root()
 
-        assert svn_url.lower().startswith(self.svn_root_url.lower())
-        self._branch_path = urllib.unquote(svn_url[len(self.svn_root_url):])
+        assert self.svn_url.lower().startswith(self.svn_root_url.lower())
+        self._branch_path = urllib.unquote(self.svn_url[len(self.svn_root_url):])
 
     def break_lock(self):
         pass
@@ -220,14 +249,27 @@ class SvnRemoteAccess(ControlDir):
         :param use_existing_dir: Use an existing directory if one exists.
         :param no_tree: If set to true prevents creation of a working tree.
         """
+        if stacked_on is not None:
+            raise errors.UnstackableBranchFormat(
+                self._format.get_branch_format(), self.user_url)
         if create_prefix:
             transport.create_prefix()
-        if not use_existing_dir:
+        elif not use_existing_dir:
             transport.mkdir(".")
         target = SvnRemoteFormat().initialize_on_transport(transport)
         target_repo = target.open_repository()
-        source_repo = self.open_repository()
-        target_repo.fetch(source_repo, revision_id=revision_id)
+        source_repo = self.find_repository()
+        # FIXME: This should ideally use the same mechanism as svnsync,
+        # or at least copy all colocated branches, too.
+        try:
+            branch = self.open_branch()
+        except errors.NotBranchError:
+            target_repo.fetch(source_repo)
+        else:
+            if revision_id is None:
+                revision_id = branch.last_revision()
+            target_repo.fetch(source_repo, revision_id=revision_id)
+            target.push_branch(branch, revision_id=revision_id)
         return target
 
     def sprout(self, url, revision_id=None, force_new_repo=False,
@@ -248,25 +290,46 @@ class SvnRemoteAccess(ControlDir):
         cloning_format = self.cloning_metadir()
         # Create/update the result branch
         result = cloning_format.initialize_on_transport(target_transport)
-        source_branch = self.open_branch()
+
         source_repository = self.find_repository()
-        try:
-            result_repo = result.find_repository()
-        except errors.NoRepositoryPresent:
+        if force_new_repo:
             result_repo = result.create_repository()
             target_is_empty = True
         else:
-            target_is_empty = None # Unknown
+            try:
+                result_repo = result.find_repository()
+            except errors.NoRepositoryPresent:
+                result_repo = result.create_repository()
+                target_is_empty = True
+            else:
+                target_is_empty = None # Unknown
         if stacked:
-            raise errors.IncompatibleRepositories(source_repository, result_repo)
+            raise errors.UnstackableBranchFormat(self._format.get_branch_format(),
+                self.root_transport.base)
         interrepo = InterRepository.get(source_repository, result_repo)
-        if revision_id is None:
-            revision_id = source_branch.last_revision()
+        try:
+            source_branch = self.open_branch()
+        except errors.NotBranchError:
+            source_branch = None
+            project = None
+            mapping = None
+        else:
+            project = source_branch.project
+            mapping = source_branch.mapping
         interrepo.fetch(revision_id=revision_id,
-            project=source_branch.project, mapping=source_branch.mapping,
-            target_is_empty=target_is_empty)
-        result_branch = source_branch.sprout(result,
-            revision_id=revision_id, repository=result_repo)
+            project=project, mapping=mapping,
+            target_is_empty=target_is_empty,
+            exclude_non_mainline=False)
+        if source_branch is not None:
+            if revision_id is None:
+                revision_id = source_branch.last_revision()
+            result_branch = source_branch.sprout(result,
+                revision_id=revision_id, repository=result_repo)
+            interbranch = InterBranch.get(source_branch, result_branch)
+            interbranch.fetch(stop_revision=revision_id,
+                    exclude_non_mainline=False) # For the tags
+        else:
+            result_branch = result.create_branch()
         if (create_tree_if_local and isinstance(target_transport, LocalTransport)
             and (result_repo is None or result_repo.make_working_trees())):
             result.create_workingtree(accelerator_tree=accelerator_tree,
@@ -288,6 +351,9 @@ class SvnRemoteAccess(ControlDir):
             return SvnRepository(self, self.root_transport)
         raise NoSvnRepositoryPresent(self.root_transport.base)
 
+    def _find_or_create_repository(self, force_new_repo=False):
+        return self.find_repository()
+
     def find_repository(self, _ignore_branch_path=False):
         """Open the repository associated with this ControlDir.
 
@@ -342,6 +408,8 @@ class SvnRemoteAccess(ControlDir):
         try:
             if stop_revision is None:
                 stop_revision = source.last_revision()
+            if stop_revision == NULL_REVISION:
+                return self.create_branch()
             relpath = self._determine_relpath(name)
             target_branch_path = relpath.lstrip("/")
             repos = self.find_repository()
@@ -381,7 +449,8 @@ class SvnRemoteAccess(ControlDir):
             else:
                 raise errors.NoColocatedBranchSupport(layout)
 
-    def create_branch(self, branch_name=None, repository=None, mapping=None):
+    def create_branch(self, branch_name=None, repository=None, mapping=None,
+            lossy=False, append_revisions_only=None):
         """See ControlDir.create_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         from bzrlib.plugins.svn.push import (
@@ -392,28 +461,44 @@ class SvnRemoteAccess(ControlDir):
         if repository is None:
             repository = self.find_repository()
 
-        if mapping is None:
-            mapping = repository.get_mapping()
+        repository.lock_write()
+        try:
+            if mapping is None:
+                mapping = repository.get_mapping()
 
-        relpath = self._determine_relpath(branch_name).strip("/")
-        if relpath == "":
-            if repository.get_latest_revnum() > 0:
-                # Bail out if there are already revisions in this repository
+            relpath = self._determine_relpath(branch_name).strip("/")
+            if relpath == "":
+                if repository.get_latest_revnum() > 0:
+                    # Bail out if there are already revisions in this repository
+                    raise errors.AlreadyBranchError(repository.transport.base)
+                # TODO: Set NULL_REVISION in SVN_PROP_BZR_BRANCHING_SCHEME on rev0
+            bp_parts = relpath.split("/")
+            existing_bp_parts = check_dirs_exist(repository.transport, bp_parts,
+                -1)
+            if len(existing_bp_parts) == len(bp_parts) and relpath != "":
                 raise errors.AlreadyBranchError(repository.transport.base)
-            # TODO: Set NULL_REVISION in SVN_PROP_BZR_BRANCHING_SCHEME on rev0
-        bp_parts = relpath.split("/")
-        existing_bp_parts = check_dirs_exist(repository.transport, bp_parts,
-            -1)
-        if len(existing_bp_parts) == len(bp_parts) and relpath != "":
-            raise errors.AlreadyBranchError(repository.transport.base)
-        if len(existing_bp_parts) < len(bp_parts)-1:
-            create_branch_container(repository.transport, relpath, "/".join(existing_bp_parts))
-        if relpath != "":
-            create_branch_with_hidden_commit(repository, relpath, NULL_REVISION)
-        return SvnBranch(repository, self, relpath, mapping)
+            if len(existing_bp_parts) < len(bp_parts)-1:
+                create_branch_container(repository.transport, relpath, "/".join(existing_bp_parts))
+            if relpath != "":
+                create_branch_with_hidden_commit(repository, relpath, NULL_REVISION,
+                        set_metadata=(not lossy))
+            branch = SvnBranch(repository, self, relpath, mapping)
+            if append_revisions_only == False:
+                branch.set_append_revisions_only(False)
+            return branch
+        finally:
+            repository.unlock()
+
+    def get_branch_reference(self, name=None):
+        """See ControlDir.get_branch_reference()."""
+        # No branch is a reference branch, but we should
+        # still raise the appropriate errors if there is no
+        # branch with the specified name.
+        self.open_branch()
+        return None
 
     def open_branch(self, name=None, unsupported=True, ignore_fallbacks=False,
-            mapping=None, branch_path=None, repository=None):
+            mapping=None, branch_path=None, repository=None, revnum=None):
         """See ControlDir.open_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         if branch_path is None:
@@ -422,10 +507,14 @@ class SvnRemoteAccess(ControlDir):
             repository = self.find_repository()
         if mapping is None:
             mapping = repository.get_mapping()
-        return SvnBranch(repository, self, branch_path, mapping)
+        return SvnBranch(repository, self, branch_path, mapping, revnum=revnum)
 
-    def create_repository(self, shared=False, format=None):
+    def create_repository(self, shared=None, format=None):
         """See ControlDir.create_repository."""
+        if shared == False:
+            from bzrlib.plugins.svn.repository import SvnRepositoryFormat
+            raise errors.IncompatibleFormat(
+                SvnRepositoryFormat(), self._format)
         return self.open_repository()
 
     def push_branch(self, source, revision_id=None, overwrite=False,
@@ -453,8 +542,12 @@ class SvnRemoteAccess(ControlDir):
             ret.target_branch = self.import_branch(source, revision_id,
                 overwrite=overwrite)
             ret.target_branch_path = "/" + ret.target_branch.get_branch_path()
-            ret.tag_conflicts = source.tags.merge_to(ret.target_branch.tags,
+            tag_ret = source.tags.merge_to(ret.target_branch.tags,
                 overwrite)
+            if isinstance(tag_ret, tuple):
+                (ret.tag_updates, ret.tag_conflicts) = tag_ret
+            else:
+                ret.tag_conflicts = tag_ret
             if source.get_push_location() is None or remember:
                 source.set_push_location(ret.target_branch.base)
         return ret
@@ -463,12 +556,13 @@ class SvnRemoteAccess(ControlDir):
         relpath = self._determine_relpath(branch_name)
         if relpath == "":
             raise errors.UnsupportedOperation(self.destroy_branch, self)
-        conn = self.root_transport.get_connection()
+        dirname, basename = urlutils.split(relpath)
+        conn = self.root_transport.get_connection(dirname.strip("/"))
         try:
             ce = conn.get_commit_editor({"svn:log": "Remove branch."})
             try:
                 root = ce.open_root()
-                root.delete_entry(".")
+                root.delete_entry(basename)
                 root.close()
             except:
                 ce.abort()
