@@ -80,6 +80,7 @@ from bzrlib.plugins.svn.layout.guess import (
     )
 from bzrlib.plugins.svn.mapping import (
     SVN_REVPROP_BZR_SIGNATURE,
+    SVN_REVPROP_BZR_TIMESTAMP,
     BzrSvnMapping,
     foreign_vcs_svn,
     find_mappings_fileprops,
@@ -87,6 +88,7 @@ from bzrlib.plugins.svn.mapping import (
     is_bzr_revision_revprops,
     mapping_registry,
     parse_svn_dateprop,
+    unpack_highres_date,
     )
 from bzrlib.plugins.svn.parents import (
     DiskCachingParentsProvider,
@@ -125,50 +127,6 @@ class DummyLockableFiles(object):
     def __init__(self, transport):
         self._transport = transport
 
-    def create_lock(self):
-        pass
-
-    def break_lock(self):
-        raise NotImplementedError(self.break_lock)
-
-    def leave_in_place(self):
-        raise NotImplementedError(self.leave_in_place)
-
-    def dont_leave_in_place(self):
-        raise NotImplementedError(self.dont_leave_in_place)
-
-    def lock_write(self, token=None):
-        if token is not None:
-            raise errors.TokenLockingNotSupported(self)
-        return SubversionRepositoryLock()
-
-    def lock_read(self):
-        return SubversionRepositoryLock()
-
-    def unlock(self):
-        pass
-
-    def is_locked(self):
-        """Return true if this LockableFiles group is locked"""
-        return False
-
-    def get_physical_lock_status(self):
-        """Return physical lock status.
-
-        Returns true if a lock is held on the transport. If no lock is held, or
-        the underlying locking mechanism does not support querying lock
-        status, false is returned.
-        """
-        return False
-
-    def get_transaction(self):
-        """Return the current active transaction.
-
-        If no transaction is active, this returns a passthrough object
-        for which all data is immediately flushed and no caching happens.
-        """
-        return transactions.PassThroughTransaction()
-
 
 class SvnRepositoryFormat(RepositoryFormat):
     """Repository format for Subversion repositories (accessed using svn_ra).
@@ -184,6 +142,8 @@ class SvnRepositoryFormat(RepositoryFormat):
     supports_external_lookups = False
     revision_graph_can_have_wrong_parents = False
     supports_versioned_directories = True
+    supports_nesting_repositories = False
+    supports_unreferenced_revisions = False
 
     @property
     def _matchingbzrdir(self):
@@ -301,27 +261,27 @@ class SubversionRepositoryCheckResult(branch.BranchCheckResult):
             self.hidden_rev_cnt += 1
 
         mapping.check_fileprops(revmeta.get_changed_fileprops(), self)
-        mapping.check_revprops(revmeta.revprops, self)
+        mapping.check_revprops(revmeta.metarev.revprops, self)
 
-        for parent_revid in revmeta.get_rhs_parents(mapping):
+        lhs_parent_revmeta = revmeta.get_lhs_parent_revmeta(mapping)
+        for parent_revid in revmeta.get_rhs_parents(mapping, lhs_parent_revmeta):
             if not self.repository.has_revision(parent_revid):
                 self.ghost_revisions.add(parent_revid)
 
         # TODO: Check that stored revno matches actual revision number
 
         # Check all paths are under branch root
-        branch_root = mapping.get_branch_root(revmeta.revprops)
+        branch_root = mapping.get_branch_root(revmeta.metarev.revprops)
         if branch_root is not None:
-            for p in revmeta.paths:
+            for p in revmeta.metarev.paths:
                 if not changes.path_is_child(branch_root, p):
                     self.paths_not_under_branch_root += 1
 
-        original_uuid = mapping.get_repository_uuid(revmeta.revprops)
+        original_uuid = mapping.get_repository_uuid(revmeta.metarev.revprops)
         if (original_uuid is not None and
             original_uuid != self.repository.uuid):
             self.different_uuid_cnt += 1
 
-        lhs_parent_revmeta = revmeta.get_lhs_parent_revmeta(mapping)
         if lhs_parent_revmeta is not None:
             lhs_parent_mapping = lhs_parent_revmeta.get_original_mapping()
             if lhs_parent_mapping is None:
@@ -334,22 +294,22 @@ class SubversionRepositoryCheckResult(branch.BranchCheckResult):
             if revmeta.get_stored_lhs_parent_revid(mapping) not in (
                     None, expected_lhs_parent_revid):
                 self.inconsistent_stored_lhs_parent += 1
-        self.check_texts(revmeta, mapping)
+        self.check_texts(revmeta, mapping, lhs_parent_revmeta)
 
-    def check_texts(self, revmeta, mapping):
+    def check_texts(self, revmeta, mapping, lhs_parent_revmeta):
         # Check for inconsistencies in text file ids/revisions
         text_revisions = revmeta.get_text_revisions(mapping)
         text_ids = revmeta.get_fileid_overrides(mapping)
         fileid_map = self.repository.get_fileid_map(revmeta, mapping)
-        path_changes = revmeta.paths
+        path_changes = revmeta.metarev.paths
         for path in set(text_ids.keys() + text_revisions.keys()):
-            full_path = urlutils.join(revmeta.branch_path, path)
+            full_path = urlutils.join(revmeta.metarev.branch_path, path)
             # TODO Check consistency against parent data
         ghost_parents = False
         parent_revmetas = []
         parent_mappings = []
         parent_fileid_maps = []
-        for revid in revmeta.get_parent_ids(mapping):
+        for revid in revmeta.get_parent_ids(mapping, lhs_parent_revmeta):
             try:
                 parent_revmeta, parent_mapping = self.repository._get_revmeta(
                     revid)
@@ -369,8 +329,11 @@ class SubversionRepositoryCheckResult(branch.BranchCheckResult):
             parent_text_revisions = []
             for parent_fileid_map, parent_mapping in zip(parent_fileid_maps,
                     parent_mappings):
-                parent_text_revisions.append(
-                    parent_fileid_map.reverse_lookup(parent_mapping, fileid))
+                try:
+                    parent_text_revisions.append(
+                        parent_fileid_map.reverse_lookup(parent_mapping, fileid))
+                except KeyError:
+                    pass
             if (text_revision != revmeta.get_revision_id(mapping) and
                     not ghost_parents and
                     not text_revision in parent_text_revisions):
@@ -385,7 +348,7 @@ class SvnRepository(ForeignRepository):
 
     chk_bytes = None
 
-    def __init__(self, bzrdir, transport, branch_path=None):
+    def __init__(self, some_dir, transport, branch_path=None):
         from bzrlib.plugins.svn import lazy_register_optimizers
         lazy_register_optimizers()
         self.vcs = foreign_vcs_svn
@@ -394,8 +357,8 @@ class SvnRepository(ForeignRepository):
         assert isinstance(transport, Transport)
 
         control_files = DummyLockableFiles(transport)
-        Repository.__init__(self, SvnRepositoryFormat(), bzrdir, control_files)
-
+        Repository.__init__(self, SvnRepositoryFormat(), some_dir, control_files)
+        self._transport = transport
         self._cached_revnum = None
         self._lock_mode = None
         self._lock_count = 0
@@ -459,6 +422,7 @@ class SvnRepository(ForeignRepository):
 
         self._parents_provider = graph.CachingParentsProvider(
             self._real_parents_provider)
+        self._parents_provider.disable_cache()
         self.texts = None
         self.revisions = None
         self.inventories = None
@@ -469,6 +433,29 @@ class SvnRepository(ForeignRepository):
         self._revmeta_provider = revmeta.RevisionMetadataProvider(self,
                 self.revinfo_cache is not None)
 
+    def _get_bzrdir(self):
+        # This is done lazily since the actual repository root may not be
+        # accessible, and we don't really need it.
+        if self._some_controldir.svn_root_url == self._some_controldir.svn_url:
+            return self._some_controldir
+        from bzrlib.plugins.svn.remote import SvnRemoteAccess
+        self._some_controldir = SvnRemoteAccess(self._some_controldir.root_transport.clone_root())
+        return self._some_controldir
+
+    def _set_bzrdir(self, value):
+        self._some_controldir = value
+
+    bzrdir = property(_get_bzrdir, _set_bzrdir)
+
+    def break_lock(self):
+        raise NotImplementedError(self.break_lock)
+
+    def dont_leave_lock_in_place(self):
+        raise NotImplementedError(self.dont_leave_lock_in_place)
+
+    def leave_lock_in_place(self):
+        raise NotImplementedError(self.leave_lock_in_place)
+
     @needs_read_lock
     def get_file_graph(self):
         """Return the graph walker for text revisions."""
@@ -491,6 +478,18 @@ class SvnRepository(ForeignRepository):
         # DEPRECATED in bzr 2.4
         return False
 
+    def pack(self, hint=None, clean_obsolete_packs=False):
+        try:
+            local_path = self.transport.local_abspath(".")
+        except bzr_errors.NotLocalUrl:
+            pass
+        else:
+            from subvertpy import repos
+            repo = repos.Repository(local_path)
+            pack_fs = getattr(repo, "pack_fs", None)
+            if pack_fs is not None:
+                pack_fs()
+
     def get_transaction(self):
         """See Repository.get_transaction()."""
         if self._write_group is None:
@@ -505,6 +504,7 @@ class SvnRepository(ForeignRepository):
         else:
             self._lock_mode = 'r'
             self._lock_count = 1
+            self._parents_provider.enable_cache()
         return self
 
     @only_raises(bzr_errors.LockNotHeld, bzr_errors.LockBroken)
@@ -522,6 +522,7 @@ class SvnRepository(ForeignRepository):
         if self._lock_count == 0:
             self._lock_mode = None
             self._clear_cached_state()
+            self._parents_provider.disable_cache()
 
     def reconcile(self, other=None, thorough=False):
         """Reconcile this repository."""
@@ -530,13 +531,24 @@ class SvnRepository(ForeignRepository):
         reconciler.reconcile()
         return reconciler
 
-    def _clear_cached_state(self, revnum=None):
+    def _cache_add_new_revision(self, revnum, revid, parents):
+        assert self.is_locked()
+        assert type(parents) == tuple or parents is None
+        self._cached_revnum = max(revnum, self._cached_revnum)
+        if parents == () and revid != NULL_REVISION:
+            parents = (NULL_REVISION,)
+        parents_cache = self._parents_provider._cache
+        if parents is None:
+            if revid in parents_cache and parents_cache[revid] is None:
+                del parents_cache[revid]
+        else:
+            parents_cache[revid] = parents
+
+    def _clear_cached_state(self):
         self._cached_tags = {}
-        self._cached_revnum = revnum
+        self._cached_revnum = None
         self._layout = None
         self._layout_source = None
-        self._parents_provider = graph.CachingParentsProvider(
-            self._real_parents_provider)
         if self._cache_obj is not None:
             self._cache_obj.commit()
 
@@ -548,6 +560,7 @@ class SvnRepository(ForeignRepository):
         else:
             self._lock_mode = 'w'
             self._lock_count = 1
+            self._parents_provider.enable_cache()
         return SubversionRepositoryLock(self)
 
     def is_write_locked(self):
@@ -562,18 +575,23 @@ class SvnRepository(ForeignRepository):
 
         Will be cached when there is a read lock open.
         """
-        if self._lock_mode in ('r','w') and self._cached_revnum is not None:
+        if self._cached_revnum is not None:
             return self._cached_revnum
-        self._cached_revnum = self.transport.get_latest_revnum()
-        return self._cached_revnum
+        revnum = self.transport.get_latest_revnum()
+        if self.is_locked():
+            self._cached_revnum = revnum
+        return revnum
 
     @needs_read_lock
     def gather_stats(self, revid=None, committers=None):
         """See Repository.gather_stats()."""
         result = {}
-        def revdate(revnum):
-            revprops = self._log.revprop_list(revnum)
-            return parse_svn_dateprop(revprops[subvertpy.properties.PROP_REVISION_DATE])
+        def check_timestamp(timestamp):
+            if not 'latestrev' in result:
+                result['latestrev'] = timestamp
+            result['firstrev'] = timestamp
+
+        count = 0
         if committers is not None and revid is not None:
             all_committers = set()
             graph = self.get_graph()
@@ -582,11 +600,27 @@ class SvnRepository(ForeignRepository):
             for rev in self.get_revisions(revids):
                 if rev.committer != '':
                     all_committers.add(rev.committer)
+                check_timestamp((rev.timestamp, rev.timezone))
             result['committers'] = len(all_committers)
-        result['firstrev'] = revdate(0)
-        result['latestrev'] = revdate(self.get_latest_revnum())
+            count = len(list(self.all_revision_ids()))
+        else:
+            mapping = self.get_mapping()
+            for revmeta in self._revmeta_provider.iter_all_revisions(
+                    self.get_layout(), mapping.is_branch_or_tag,
+                    self.get_latest_revnum()):
+                if revmeta.is_hidden(mapping):
+                    continue
+                try:
+                    timestamp = unpack_highres_date(
+                        revmeta.metarev.revprops[SVN_REVPROP_BZR_TIMESTAMP])
+                except KeyError:
+                    timestamp = parse_svn_dateprop(
+                        revmeta.metarev.revprops[
+                            subvertpy.properties.PROP_REVISION_DATE])
+                check_timestamp(timestamp)
+                count += 1
         # Approximate number of revisions
-        result['revisions'] = self.get_latest_revnum()+1
+        result['revisions'] = count
         result['svn-uuid'] = self.uuid
         result['svn-last-revnum'] = self.get_latest_revnum()
         return result
@@ -630,15 +664,15 @@ class SvnRepository(ForeignRepository):
         return self._parents_provider
 
     def get_deltas_for_revisions(self, revisions):
-        """See Repository.get_deltas_for_revisions()."""
         """See Repository.get_deltas_for_revisions()."""
         for revision in revisions:
             yield self.get_delta_for_revision(revision)
 
-    def get_revision_delta(self, revid):
-        return self.get_delta_for_revision(self.get_revision(revid))
+    def get_revision_delta(self, revid, specific_fileids=None):
+        return self.get_delta_for_revision(self.get_revision(revid),
+            specific_fileids=specific_fileids)
 
-    def get_delta_for_revision(self, revision):
+    def get_delta_for_revision(self, revision, specific_fileids=None):
         """See Repository.get_delta_for_revision()."""
         parentrevmeta = revision.svn_meta.get_lhs_parent_revmeta(
             revision.mapping)
@@ -656,7 +690,7 @@ class SvnRepository(ForeignRepository):
             start_empty = False
         editor = TreeDeltaBuildEditor(revision.svn_meta, revision.mapping,
             self.get_fileid_map(revision.svn_meta, revision.mapping),
-            parentfileidmap)
+            parentfileidmap, specific_fileids=specific_fileids)
         conn = self.transport.get_connection(parent_branch_path)
         try:
             reporter = conn.do_diff(revision.svn_meta.metarev.revnum, "",
@@ -671,15 +705,9 @@ class SvnRepository(ForeignRepository):
                 raise
         finally:
             self.transport.add_connection(conn)
+        editor.sort()
         return editor.delta
 
-    def _annotate(self, path, revnum, fileid, revid, mapping):
-        assert type(path) is str
-        from bzrlib.plugins.svn.annotate import Annotater
-        annotater = Annotater(self, mapping, fileid, revid)
-        annotater.check_file_revs(path, revnum)
-        return annotater.get_annotated()
-
     def set_layout(self, layout):
         """Set the layout that should be used by default by this instance."""
         self.get_mapping().check_layout(self, layout)
@@ -765,12 +793,18 @@ class SvnRepository(ForeignRepository):
 
     def _check(self, revision_ids=None, callback_refs=None, check_repo=True):
         ret = SubversionRepositoryCheckResult(self)
+        # FIXME: If this is a local repository, call out to subvertpy.repos.verify_fs.
+        # bug=843944
         pb = ui.ui_factory.nested_progress_bar()
         try:
             if revision_ids is not None:
-                for i, revid in enumerate(revision_ids):
+                for i, (revid, revmeta, mapping) in enumerate(
+                        self._iter_revmetas(revision_ids)):
+                    if revmeta is None:
+                        continue
+                    if revid == NULL_REVISION:
+                        continue
                     pb.update("checking revisions", i, len(revision_ids))
-                    revmeta, mapping = self._get_revmeta(revid)
                     ret.check_revmeta(revmeta)
             else:
                 last_revnum = self.get_latest_revnum()
@@ -778,7 +812,7 @@ class SvnRepository(ForeignRepository):
                         self.get_layout(), self.get_mapping().is_branch_or_tag,
                         last_revnum):
                     pb.update("checking revisions",
-                        last_revnum-revmeta.revnum, last_revnum)
+                        last_revnum-revmeta.metarev.revnum, last_revnum)
                     ret.check_revmeta(revmeta)
         finally:
             pb.finished()
@@ -803,6 +837,7 @@ class SvnRepository(ForeignRepository):
                if part of the ancestry of valid revisions) won't be
                returned.
         """
+        ret = set()
         if mapping is None:
             mapping = self.get_mapping()
         if layout is None:
@@ -811,11 +846,12 @@ class SvnRepository(ForeignRepository):
                 mapping.is_branch_or_tag, self.get_latest_revnum()):
             if revmeta.is_hidden(mapping):
                 continue
-            yield revmeta.get_revision_id(mapping)
+            ret.add(revmeta.get_revision_id(mapping))
+        return ret
 
     def set_make_working_trees(self, new_value):
         """See Repository.set_make_working_trees()."""
-        pass # ignored, nowhere to store it...
+        raise bzr_errors.UnsupportedOperation(self.set_make_working_trees, self)
 
     def make_working_trees(self):
         """See Repository.make_working_trees().
@@ -831,7 +867,15 @@ class SvnRepository(ForeignRepository):
         pending = set(keys)
         parent_map = {}
         while pending:
-            this_parent_map = self.get_parent_map(pending)
+            this_parent_map = {}
+            for revid in pending:
+                if revid == NULL_REVISION:
+                    continue
+                parents = self._get_parents(revid)
+                if parents is not None:
+                    if tuple(parents) == (NULL_REVISION,):
+                        parents = ()
+                    this_parent_map[revid] = parents
             parent_map.update(this_parent_map)
             pending = set()
             map(pending.update, this_parent_map.itervalues())
@@ -871,6 +915,19 @@ class SvnRepository(ForeignRepository):
         else:
             return (kind == subvertpy.NODE_DIR)
 
+    def iter_files_bytes(self, desired_files):
+        per_revision = {}
+        for (file_id, revision_id, identifier) in desired_files:
+            per_revision.setdefault(revision_id, []).append(
+                (file_id, identifier))
+        for revid, files in per_revision.iteritems():
+            try:
+                tree = self.revision_tree(revid)
+            except bzr_errors.NoSuchRevision:
+                raise bzr_errors.RevisionNotPresent(revid, self)
+            for identifier, curfile in tree.iter_files_bytes(files):
+                yield identifier, curfile
+
     def revision_trees(self, revids):
         """See Repository.revision_trees()."""
         # TODO: Use diffs
@@ -879,29 +936,51 @@ class SvnRepository(ForeignRepository):
 
     def revision_tree(self, revision_id):
         """See Repository.revision_tree()."""
-        if revision_id in (NULL_REVISION, None):
+        if ensure_null(revision_id) == NULL_REVISION:
             inventory = Inventory(root_id=None)
             inventory.revision_id = revision_id
             return InventoryRevisionTree(self, inventory, revision_id)
 
         return SvnRevisionTree(self, revision_id)
 
+    def _get_parents(self, revid):
+        try:
+            revmeta, mapping = self._get_revmeta(ensure_null(revid))
+        except bzr_errors.NoSuchRevision:
+            return None
+        else:
+            parentrevmeta = revmeta.get_lhs_parent_revmeta(mapping)
+            return revmeta.get_parent_ids(mapping, parentrevmeta)
+
     def get_parent_map(self, revids):
         """See Repository.get_parent_map()."""
         parent_map = {}
         for revision_id in revids:
+            if type(revision_id) != str:
+                raise ValueError("expected sequence of revision ids")
             if revision_id == NULL_REVISION:
                 parent_map[revision_id] = ()
                 continue
-            try:
-                revmeta, mapping = self._get_revmeta(ensure_null(revision_id))
-            except bzr_errors.NoSuchRevision:
+            parents = self._get_parents(revision_id)
+            if parents is None:
                 continue
-            else:
-                parentrevmeta = revmeta.get_lhs_parent_revmeta(mapping)
-                parent_map[revision_id] = revmeta.get_parent_ids(mapping, parentrevmeta)
+            if len(parents) == 0:
+                parents = [NULL_REVISION]
+            parent_map[revision_id] = tuple(parents)
         return parent_map
 
+    def _iter_revmetas(self, revision_ids):
+        for revid in revision_ids:
+            if revid == NULL_REVISION:
+                yield (NULL_REVISION, None, None)
+            else:
+                try:
+                    (revmeta, mapping) = self._get_revmeta(revid)
+                except bzr_errors.NoSuchRevision:
+                    yield (revid, None, None)
+                else:
+                    yield (revid, revmeta, mapping)
+
     def _get_revmeta(self, revision_id):
         foreign_revid, mapping = self.lookup_bzr_revision_id(revision_id)
         (uuid, branch, revnum) = foreign_revid
@@ -913,7 +992,6 @@ class SvnRepository(ForeignRepository):
         if not revision_id or not isinstance(revision_id, str):
             raise bzr_errors.InvalidRevisionId(revision_id=revision_id,
                 branch=self)
-
         revmeta, mapping = self._get_revmeta(revision_id)
         parentrevmeta = revmeta.get_lhs_parent_revmeta(mapping)
         return revmeta.get_revision(mapping, parentrevmeta)
@@ -1072,7 +1150,7 @@ class SvnRepository(ForeignRepository):
         try:
             for project, bp, nick, has_props, revnum in layout.get_branches(self,
                     revnum):
-                branches.append(SvnBranch(self, None, bp, mapping,
+                branches.append(SvnBranch(self, self.bzrdir, bp, mapping,
                     _skip_check=True, revnum=revnum))
         finally:
             pb.finished()
@@ -1106,6 +1184,7 @@ class SvnRepository(ForeignRepository):
         ret = {}
         for (project, branch_path, name, has_props, revnum) in layout.get_tags(self,
                 revnum, project=project):
+            assert type(name) is unicode
             base_revmeta = self._revmeta_provider.lookup_revision(branch_path, revnum)
             ret[name] = base_revmeta.get_tag_revmeta(mapping)
 
@@ -1133,6 +1212,8 @@ class SvnRepository(ForeignRepository):
                            timezone=None, committer=None, revprops=None,
                            revision_id=None, lossy=False):
         """See Repository.get_commit_builder()."""
+        if self._lock_mode != 'w':
+            raise bzr_errors.NotWriteLocked(self)
         from bzrlib.plugins.svn.commit import SvnCommitBuilder
         if branch is None:
             raise Exception("branch option is required for "
@@ -1140,35 +1221,42 @@ class SvnRepository(ForeignRepository):
         append_revisions_only = branch.get_config().get_append_revisions_only()
         if append_revisions_only is None:
             append_revisions_only = True
-        last_revmeta, last_mapping = branch.last_revmeta()
+        base_revmeta, base_mapping = branch.last_revmeta(skip_hidden=False)
         if len(parents) == 0 or tuple(parents) == (NULL_REVISION,):
             base_foreign_revid = None
-            base_mapping = None
-            if last_revmeta is None:
+            if base_revmeta is None:
                 root_action = ("create", )
+                base_mapping = None
                 branch_path = branch._branch_path
+            elif branch.last_revision() == NULL_REVISION:
+                root_action = ("open", base_revmeta.metarev.revnum)
+                branch_path = base_revmeta.metarev.branch_path
             else:
-                root_action = ("replace", last_revmeta.metarev.revnum)
-                branch_path = last_revmeta.metarev.branch_path
+                root_action = ("replace", base_revmeta.metarev.revnum)
+                branch_path = base_revmeta.metarev.branch_path
         else:
-            base_foreign_revid, base_mapping = \
-                self.lookup_bzr_revision_id(parents[0], project=branch.project)
-            branch_path = last_revmeta.metarev.branch_path
-            if ((base_foreign_revid[2] != last_revmeta.metarev.revnum or
-                base_foreign_revid[1] != last_revmeta.metarev.branch_path)):
-                root_action = ("replace", last_revmeta.metarev.revnum)
+            branch_path = base_revmeta.metarev.branch_path
+
+            if parents[0] != branch.last_revision():
+                root_action = ("replace", base_revmeta.metarev.revnum)
+                base_foreign_revid, base_mapping = \
+                     self.lookup_bzr_revision_id(parents[0], project=branch.project)
             else:
-                root_action = ("open", )
+                root_action = ("open", base_revmeta.metarev.revnum)
+                base_foreign_revid = base_revmeta.metarev.get_foreign_revid()
 
         if root_action[0] == "replace" and append_revisions_only:
             raise bzr_errors.AppendRevisionsOnlyViolation(branch.base)
 
-        return SvnCommitBuilder(self, branch_path, parents,
+        ret = SvnCommitBuilder(self, branch_path, parents,
                                 config, timestamp, timezone, committer,
                                 revprops, revision_id,
                                 base_foreign_revid, base_mapping,
                                 root_action=root_action,
-                                push_metadata=not lossy)
+                                push_metadata=not lossy,
+                                branch=branch)
+        self.start_write_group()
+        return ret
 
     def find_fileprop_paths(self, layout, from_revnum, to_revnum,
                                project=None, check_removed=False):
@@ -1299,5 +1387,7 @@ def find_tags_between(revmeta_provider, project, layout, mapping, from_revnum,
         # Layout wasn't able to determine tag name from path
         if name is None:
             continue
+        name = name.decode("utf-8")
+        assert type(name) is unicode
         ret[name] = revmeta
     return ret
