@@ -25,7 +25,10 @@ from bzrlib.tests import KnownFailure
 from bzrlib.tests.blackbox import ExternalBase
 
 from bzrlib.plugins.svn.convert import load_dumpfile
-from bzrlib.plugins.svn.layout.standard import RootLayout
+from bzrlib.plugins.svn.layout.standard import (
+    RootLayout,
+    TrunkLayout,
+    )
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
 
@@ -39,8 +42,8 @@ class TestBranch(SubversionTestCase, ExternalBase):
         ExternalBase.tearDown(self)
 
     def test_branch_empty(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr("branch %s dc" % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr("branch %s dc" % branch.base)
 
     def commit_something(self, repos_url):
         dc = self.get_commit_editor(repos_url)
@@ -48,36 +51,32 @@ class TestBranch(SubversionTestCase, ExternalBase):
         dc.close()
 
     def test_branch_onerev(self):
-        repos_url = self.make_client('d', 'de')
-        self.commit_something(repos_url)
-        self.run_bzr("branch %s dc" % repos_url)
-        self.assertEquals("2\n", self.run_bzr("revno de")[0])
+        tree = self.make_svn_branch_and_tree('d', 'de')
+        self.commit_something(tree.branch.base)
+        self.run_bzr("branch %s dc" % tree.branch.base)
+        self.assertEquals("1\n", self.run_bzr("revno de")[0])
 
     def test_branch_onerev_stacked(self):
-        repos_url = self.make_client('d', 'de')
-        self.commit_something(repos_url)
-        self.run_bzr("branch --stacked %s dc" % repos_url, retcode=3)
+        tree = self.make_svn_branch_and_tree('d', 'de')
+        self.commit_something(tree.branch.base)
+        self.run_bzr("branch --stacked %s dc" % tree.branch.base, retcode=3)
 
     def test_log_empty(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr('log %s' % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr('log %s' % branch.base)
 
     def test_info_verbose(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr('info -v %s' % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr('info -v %s' % branch.base)
 
     def test_pack(self):
         repos_url = self.make_repository('d')
         self.run_bzr('pack %s' % repos_url)
 
     def test_rmbranch(self):
-        repos_url = self.make_repository('d')
+        branch = self.make_svn_branch('d')
 
-        dc = self.get_commit_editor(repos_url)
-        dc.add_dir("trunk")
-        dc.close()
-
-        self.run_bzr("rmbranch %s/trunk" % repos_url)
+        self.run_bzr("rmbranch %s" % branch.base)
 
     def test_push_create_prefix(self):
         repos_url = self.make_repository('d')
@@ -155,21 +154,20 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.assertContainsRe(output, "Reconciliation complete.\n")
 
     def test_missing(self):
-        repos_url = self.make_repository('d')
+        branch = self.make_svn_branch('d')
 
         self.run_bzr("init dc")
 
         os.chdir("dc")
-        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
-        self.assertContainsRe(output, "You are missing 1 revision\\(s\\):")
+        output, err = self.run_bzr("missing %s" % branch.base)
+        self.assertContainsRe(output, "Branches are up to date.")
 
-        dc = self.get_commit_editor(repos_url)
-        trunk = dc.add_dir('trunk')
-        trunk.add_file("trunk/foo").modify()
+        dc = self.get_commit_editor(branch.base)
+        dc.add_file("foo").modify()
         dc.close()
 
-        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
-        self.assertContainsRe(output, "You are missing 2 revision\\(s\\):")
+        output, err = self.run_bzr("missing %s" % branch.base, retcode=1)
+        self.assertContainsRe(output, "You are missing 1 revision")
 
     def test_push_overwrite(self):
         repos_url = self.make_repository('d')
@@ -188,12 +186,14 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_empty(self):
         repos_url = self.make_repository('dp')
+        Repository.open(repos_url).store_layout(RootLayout())
         self.run_bzr("init --default-rich-root dc")
         os.chdir("dc")
         self.run_bzr("dpush %s" % repos_url)
 
     def test_dpush(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -207,6 +207,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_new(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -222,6 +223,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_wt_diff(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -237,7 +239,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
                 self.run_bzr("status dc")[0])
 
     def test_info_workingtree(self):
-        repos_url = self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc')
         # Info is broken against bzr 2.3
         if bzrlib_version < (2, 4, 0):
             raise KnownFailure("bzr info against svn repositories is "
@@ -442,7 +444,7 @@ Node-copyfrom-path: x
                 self.run_bzr('info a')[0])
 
     def test_lightweight_checkout_lightweight_checkout(self):
-        repos_url = self.make_client("a", "dc")
+        self.make_svn_branch_and_tree("a", "dc")
         self.build_tree({'dc/foo': "test", 'dc/bla': "ha"})
         self.client_add("dc/foo")
         self.client_add("dc/bla")
@@ -450,11 +452,11 @@ Node-copyfrom-path: x
         self.run_bzr("checkout --lightweight dc de")
 
     def test_commit(self):
-        repos_url = self.make_client('d', 'de')
+        self.make_svn_branch_and_tree('d', 'de')
         self.build_tree({'de/foo': 'bla'})
         self.run_bzr("add de/foo")
         self.run_bzr("commit -m test de")
-        self.assertEquals("2\n", self.run_bzr("revno de")[0])
+        self.assertEquals("1\n", self.run_bzr("revno de")[0])
 
     # this method imported from bzrlib.tests.test_msgeditor:
     def make_fake_editor(self, message='test message from fed\\n'):
@@ -601,7 +603,11 @@ if len(sys.argv) == 2:
         svn_url = self.make_repository('d')
 
         self.run_bzr('svn-import --format 1.9-rich-root %s dc' % svn_url)
-        self.assertEquals("Repository branch (format: 1.14-rich-root)\nLocation:\n  shared repository: dc\n  repository branch: dc\n\nRelated branches:\n  parent branch: d\n", self.run_bzr('info dc')[0])
+        self.assertEquals(
+            "Shared repository (format: 1.14-rich-root)\n"
+            "Location:\n"
+            "  shared repository: dc\n",
+            self.run_bzr('info dc')[0])
 
     def test_svn_layout(self):
         svn_url = self.make_repository('d')
@@ -612,11 +618,11 @@ if len(sys.argv) == 2:
 
         self.assertEquals(
             'Repository root: %s\n'
-            'Layout: root\n'
-            'Branch path: \n'
-            'No tag support\n'
-            'No custom branch support\n'
-            'Push merged revisions: False\n' % svn_url,
+            'Layout: trunk-variable\n'
+            'Branch path: trunk\n'
+            'Tag container directory: tags\n'
+            'Branch container directory: branches\n'
+            'Push merged revisions: None\n' % svn_url,
                 self.run_bzr('svn-layout %s' % svn_url)[0])
 
     def test_svn_branches(self):
@@ -642,26 +648,26 @@ if len(sys.argv) == 2:
             self.run_bzr('svn-branches --layout trunk %s' % svn_url)[0])
 
     def test_diff(self):
-        repos_url = self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc')
         self.build_tree_contents([("dc/file", "bar")])
         self.client_add("dc/file")
         self.run_bzr('diff dc', retcode=True)
 
     def test_diff_svn(self):
-        repos_url = self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc')
         self.build_tree_contents([("dc/file", "bar")])
         self.client_add("dc/file")
         self.run_bzr('diff --format=svn dc', retcode=True)
 
     def test_ls(self):
-        repos_url = self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc')
         self.build_tree_contents([("dc/file", "bar")])
         self.client_add("dc/file")
         self.assertEquals("dc/file\n",
             self.run_bzr('ls dc', retcode=0)[0])
 
     def test_log_v(self):
-        repos_url = self.make_client('d', 'dc')
-        self.commit_something(repos_url)
+        tree = self.make_svn_branch_and_tree('d', 'dc')
+        self.commit_something(tree.branch.base)
         self.run_bzr('log -v d', retcode=0)
         self.run_bzr('log -v dc', retcode=0)
