@@ -88,7 +88,7 @@ class TestDPush(SubversionTestCase):
     def setUp(self):
         super(TestDPush, self).setUp()
         transport.disabled_capabilities.update(["commit-revprops", "log-revprops"])
-        self.repos_url = self.make_repository('d')
+        self.repos_url = self.make_svn_repository('d')
 
         dc = self.commit_editor()
         foo = dc.add_dir("foo")
@@ -107,7 +107,11 @@ class TestDPush(SubversionTestCase):
         return self.get_commit_editor(self.repos_url, message)
 
     def dpush(self, source, target):
-        return InterToSvnBranch(source, target)._update_revisions_lossy()
+        target.lock_write()
+        try:
+            return InterToSvnBranch(source, target)._update_revisions(lossy=True)
+        finally:
+            target.unlock()
 
     def test_change_single(self):
         self.build_tree({'dc/foo/bla': 'other data'})
@@ -117,7 +121,8 @@ class TestDPush(SubversionTestCase):
         source_branch = self.bzrdir.open_branch()
         source_branch.lock_write()
         try:
-            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+            old_revid, new_revid, revid_map = self.dpush(source_branch,
+                    self.svndir.open_branch())
         finally:
             source_branch.unlock()
 
@@ -146,7 +151,8 @@ class TestDPush(SubversionTestCase):
         source_branch = self.bzrdir.open_branch()
         source_branch.lock_write()
         try:
-            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+            old_revid, new_revid, revid_map = self.dpush(source_branch,
+                    self.svndir.open_branch())
         finally:
             source_branch.unlock()
 
@@ -175,7 +181,8 @@ class TestDPush(SubversionTestCase):
         source_branch = self.bzrdir.open_branch()
         source_branch.lock_write()
         try:
-            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+            old_revid, new_revid, revid_map = self.dpush(source_branch,
+                self.svndir.open_branch())
         finally:
             source_branch.unlock()
 
@@ -212,7 +219,7 @@ class TestPush(SubversionTestCase):
 
     def setUp(self):
         super(TestPush, self).setUp()
-        self.repos_url = self.make_repository('d')
+        self.repos_url = self.make_svn_repository('d')
 
         dc = self.commit_editor()
         foo = dc.add_dir("foo")
@@ -231,8 +238,8 @@ class TestPush(SubversionTestCase):
         bzrbranch = self.bzrdir.open_branch()
         result = svnbranch.pull(bzrbranch)
         self.assertEqual(0, result.new_revno - result.old_revno)
-        self.assertEqual(svnbranch.revision_history(),
-                         bzrbranch.revision_history())
+        self.assertEqual(svnbranch.last_revision_info(),
+                         bzrbranch.last_revision_info())
 
     def test_child(self):
         dc = self.commit_editor()
@@ -378,8 +385,8 @@ class TestPush(SubversionTestCase):
         os.mkdir("b")
         repos = self.svndir.sprout("b")
 
-        self.assertEqual(Branch.open("dc").revision_history(),
-                         Branch.open("b").revision_history())
+        self.assertEqual(Branch.open("dc").last_revision_info(),
+                         Branch.open("b").last_revision_info())
 
     def test_fetch_preserves_file_revid(self):
         self.build_tree({'dc/file': 'data'})
@@ -479,14 +486,14 @@ class TestPush(SubversionTestCase):
         self.assertTrue(tree.has_filename('file'))
         self.assertTrue(tree.has_filename('adir'))
 
-        self.assertEqual(self.svndir.open_branch().revision_history(),
-                         self.bzrdir.open_branch().revision_history())
+        self.assertEqual(self.svndir.open_branch().last_revision_info(),
+                         self.bzrdir.open_branch().last_revision_info())
 
         self.assertEqual(wt.branch.last_revision(),
                 repos.generate_revision_id(3, "", mapping))
 
     def test_multiple_diverged(self):
-        oc_url = self.make_repository("o")
+        oc_url = self.make_svn_repository("o")
 
         self.build_tree({'dc/file': 'data'})
         wt = self.bzrdir.open_workingtree()
@@ -578,7 +585,7 @@ class PushNewBranchTests(SubversionTestCase):
         return bzrwt, revid
 
     def test_fetch_after_push(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         newdir = BzrDir.open("%s/trunk" % repos_url)
         bzrwt, revid = self._create_single_rev_bzrwt()
         newbranch = newdir.import_branch(bzrwt.branch)
@@ -598,7 +605,7 @@ class PushNewBranchTests(SubversionTestCase):
             bzrwt.unlock()
 
     def test_single_revision(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         newdir = BzrDir.open("%s/trunk" % repos_url)
         bzrwt, revid = self._create_single_rev_bzrwt()
         newbranch = newdir.import_branch(bzrwt.branch)
@@ -609,10 +616,9 @@ class PushNewBranchTests(SubversionTestCase):
         finally:
             bzrwt.unlock()
         self.assertEquals(revid, newbranch.last_revision())
-        self.assertEquals([revid], newbranch.revision_history())
 
     def test_single_revision_single_branch(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
@@ -724,7 +730,7 @@ class PushNewBranchTests(SubversionTestCase):
         self.assertTrue(os.path.exists("bzrco/baz.txt"))
         target_branch = Branch.open(repos_url+"/trunk")
         self.assertRaises(AppendRevisionsOnlyViolation, wt.branch.push, target_branch)
-        target_branch.get_config().set_user_option('append_revisions_only', 'False')
+        target_branch.set_append_revisions_only(False)
         wt.branch.push(target_branch)
 
     def test_push_merge_unchanged_file(self):
@@ -733,7 +739,7 @@ class PushNewBranchTests(SubversionTestCase):
                 t.get_file_revision(t.path2id("bar.txt")))
             self.assertEquals(other_revid,
                 t.get_file_revision(t.path2id("bar2.txt")))
-        repos_url = self.make_repository("test")
+        repos_url = self.make_svn_repository("test")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -784,7 +790,7 @@ class PushNewBranchTests(SubversionTestCase):
         check_tree(cpyrepos.revision_tree(mergingrevid))
 
     def test_missing_prefix_error(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
@@ -794,7 +800,7 @@ class PushNewBranchTests(SubversionTestCase):
                           lambda: newdir.import_branch(bzrwt.branch))
 
     def test_repeat(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
@@ -802,7 +808,6 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid, newbranch.last_revision())
-        self.assertEquals([revid], newbranch.revision_history())
         self.build_tree({'c/test': "Tour de France"})
         bzrwt.commit("Do a commit")
         newdir = BzrDir.open(repos_url+"/trunk")
@@ -810,7 +815,7 @@ class PushNewBranchTests(SubversionTestCase):
                           bzrwt.branch)
 
     def test_multiple(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
@@ -820,10 +825,9 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid2, newbranch.last_revision())
-        self.assertEquals([revid1, revid2], newbranch.revision_history())
 
     def test_dato(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/foo.txt': "foo"})
         bzrwt.add("foo.txt")
@@ -835,7 +839,7 @@ class PushNewBranchTests(SubversionTestCase):
                 Repository.open(repos_url).get_revision(revid1).committer)
 
     def test_utf8_commit_msg(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/foo.txt': "foo"})
         bzrwt.add("foo.txt")
@@ -846,7 +850,7 @@ class PushNewBranchTests(SubversionTestCase):
                 Repository.open(repos_url).get_revision(revid1).message)
 
     def test_kind_change_file_to_directory(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/foo.txt': "foo"})
         bzrwt.add("foo.txt")
@@ -859,7 +863,7 @@ class PushNewBranchTests(SubversionTestCase):
         newdir.import_branch(bzrwt.branch)
 
     def test_kind_change_directory_to_file(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/foo.txt/bar': "foo"})
         bzrwt.add("foo.txt")
@@ -873,7 +877,7 @@ class PushNewBranchTests(SubversionTestCase):
         newdir.import_branch(bzrwt.branch)
 
     def test_multiple_part_exists(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir('trunk')
@@ -898,7 +902,7 @@ class PushNewBranchTests(SubversionTestCase):
             , revid1, revid2], newbranch.revision_history())
 
     def test_push_overwrite(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -922,16 +926,16 @@ class PushNewBranchTests(SubversionTestCase):
         revid2 = bzrwt2.commit("Do a commit")
 
         bzrwt1.branch.push(Branch.open(repos_url+"/trunk"))
-        self.assertEquals(bzrwt1.branch.revision_history(),
-                Branch.open(repos_url+"/trunk").revision_history())
+        self.assertEquals(bzrwt1.branch.last_revision_info(),
+                Branch.open(repos_url+"/trunk").last_revision_info())
 
         bzrwt2.branch.push(Branch.open(repos_url+"/trunk"), overwrite=True)
 
-        self.assertEquals(bzrwt2.branch.revision_history(),
-                Branch.open(repos_url+"/trunk").revision_history())
+        self.assertEquals(bzrwt2.branch.last_revision_info(),
+            Branch.open(repos_url+"/trunk").last_revision_info())
 
     def test_push_overwrite_unrelated(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -953,16 +957,16 @@ class PushNewBranchTests(SubversionTestCase):
         revid2 = bzrwt2.commit("Do a commit")
 
         bzrwt1.branch.push(Branch.open(repos_url+"/trunk"))
-        self.assertEquals(bzrwt1.branch.revision_history(),
-                Branch.open(repos_url+"/trunk").revision_history())
+        self.assertEquals(bzrwt1.branch.last_revision_info(),
+                Branch.open(repos_url+"/trunk").last_revision_info())
 
         bzrwt2.branch.push(Branch.open(repos_url+"/trunk"), overwrite=True)
 
-        self.assertEquals(bzrwt2.branch.revision_history(),
-                Branch.open(repos_url+"/trunk").revision_history())
+        self.assertEquals(bzrwt2.branch.last_revision_info(),
+                Branch.open(repos_url+"/trunk").last_revision_info())
 
     def test_complex_rename(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add("registry")
@@ -978,7 +982,6 @@ class PushNewBranchTests(SubversionTestCase):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid2, newbranch.last_revision())
-        self.assertEquals([revid1, revid2], newbranch.revision_history())
         tree = newbranch.repository.revision_tree(revid2)
         delta = tree.changes_from(bzrwt)
         self.assertFalse(delta.has_changed())
@@ -990,7 +993,7 @@ class PushNewBranchTests(SubversionTestCase):
         BzrDir.open(repos_url+"/trunk").sprout("n")
 
     def test_rename_dir_changing_contents(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add("registry", "dirid")
@@ -1038,7 +1041,7 @@ class PushNewBranchTests(SubversionTestCase):
         return bzrwt, old_ie, new_ie, foo_ie, revid1, revid2
 
     def test_change_root(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt, old_ie, new_ie, foo_ie, revid1, revid2 = self._create_bzrwt_with_changed_root()
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
@@ -1052,11 +1055,11 @@ class PushNewBranchTests(SubversionTestCase):
         self.assertEquals(tree1.path2id("foo"), foo_ie.file_id)
         self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
 
-        self.assertEquals(bzrwt.branch.revision_history(),
-                          newbranch.revision_history())
+        self.assertEquals(bzrwt.branch.last_revision_info(),
+                          newbranch.last_revision_info())
 
     def test_change_root_fetch(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         (bzrwt, old_ie, new_ie, foo_ie, revid1, revid2) = \
             self._create_bzrwt_with_changed_root()
         newdir = BzrDir.open(repos_url+"/trunk")
@@ -1083,7 +1086,7 @@ class PushNewBranchTests(SubversionTestCase):
         self.assertEquals(tree2.path2id("foo"), foo_ie.file_id)
 
     def test_rename_dir(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add("registry", "dirid")
@@ -1105,7 +1108,7 @@ class PushNewBranchTests(SubversionTestCase):
         check(copybranch)
 
     def test_push_non_lhs_parent(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add("registry")
@@ -1146,13 +1149,13 @@ class PushNewBranchTests(SubversionTestCase):
         trunk = Branch.open(repos_url + "/branches/foo")
         self.assertRaises(AppendRevisionsOnlyViolation, trunk.pull,
             bzrwt.branch)
-        trunk.get_config().set_user_option('append_revisions_only', 'False')
+        trunk.set_append_revisions_only(False)
         trunk.pull(bzrwt.branch)
 
         self.assertEquals([revid1, revid2, revid3], trunk.revision_history())
 
     def test_complex_replace_dir(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add(["registry"], ["origdir"])
@@ -1174,7 +1177,7 @@ class PushNewBranchTests(SubversionTestCase):
         BzrDir.open(repos_url+"/trunk").sprout("n")
 
     def test_subdir_becomes_branch_root(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1208,7 +1211,7 @@ class PushNewBranchTests(SubversionTestCase):
                 paths['trunk'])
 
     def test_push_pointless(self):
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1234,7 +1237,7 @@ class TestPushTwice(SubversionTestCase):
 
     def test_push_twice(self):
         # bug 208566
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1296,7 +1299,7 @@ class InterToSvnRepositoryTestCase(SubversionTestCase):
         super(InterToSvnRepositoryTestCase, self).setUp()
         self.from_bzrdir = BzrDir.create('bzrrepo')
         self.from_repo = self.from_bzrdir.create_repository(shared=True)
-        self.svn_repo_url = self.make_repository('svnrepo')
+        self.svn_repo_url = self.make_svn_repository('svnrepo')
         self.to_repo = Repository.open(self.svn_repo_url)
         self.interrepo = InterRepository.get(self.from_repo, self.to_repo)
 
@@ -1356,6 +1359,8 @@ class PushRevisionTests(InterToSvnRepositoryTestCase):
         self.revid1 = tree.commit('msg')
 
     def test_push_first_revision_with_metadata(self):
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
         self.interrepo.push_single_revision("trunk",
             self.interrepo._get_branch_config("trunk"),
             self.from_repo.get_revision(self.revid1),
@@ -1366,6 +1371,8 @@ class PushRevisionTests(InterToSvnRepositoryTestCase):
         # FIXME: Check revision properties
 
     def test_push_first_revision_without_metadata(self):
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
         self.interrepo.push_single_revision("trunk",
             self.interrepo._get_branch_config("trunk"),
             self.from_repo.get_revision(self.revid1),
@@ -1386,6 +1393,9 @@ class PushRevisionTests(InterToSvnRepositoryTestCase):
         config = self.interrepo._get_branch_config("trunk")
         rev1 = self.from_repo.get_revision(self.revid1)
 
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
+
         # With append revisions only disabled but overwrite it should work
         self.interrepo.push_single_revision("trunk", config, rev1,
             push_metadata=False, base_foreign_info=(None, None),
@@ -1406,6 +1416,9 @@ class PushRevisionTests(InterToSvnRepositoryTestCase):
         config = self.interrepo._get_branch_config("trunk")
         rev1 = self.from_repo.get_revision(self.revid1)
 
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
+
         self.interrepo.push_single_revision("trunk", config, rev1,
             push_metadata=False, base_foreign_info=(None, None), root_action=("replace", 1))
 
@@ -1442,6 +1455,8 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         config = self.interrepo._get_branch_config("trunk")
         self.addCleanup(self.from_repo.unlock)
         self.from_repo.lock_read()
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
         rev1 = self.from_repo.get_revision(self.revid1)
         self.interrepo.push_single_revision("trunk",
             config, rev1, push_metadata=True, root_action=("create", ),
@@ -1452,7 +1467,7 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         self.interrepo.push_revision_inclusive("trunk", config,
             rev_merged, push_merged=False,
             layout=TrunkLayout0(), project="",
-            push_metadata=True, root_action=("open",),
+            push_metadata=True, root_action=("open", self.to_repo.get_latest_revnum()),
             base_foreign_info=foreign_rev_info)
         log = self.client_log(self.svn_repo_url, 2, 0)
         self.assertEquals(log[1][0],
@@ -1464,6 +1479,8 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         config = self.interrepo._get_branch_config("trunk")
         self.addCleanup(self.from_repo.unlock)
         self.from_repo.lock_read()
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
         rev1 = self.from_repo.get_revision(self.revid1)
         self.interrepo.push_single_revision("trunk",
             config, rev1, push_metadata=True, root_action=("create", ),
@@ -1472,7 +1489,7 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         self.interrepo.push_revision_inclusive("trunk", config,
             rev_merged, push_merged=True,
             layout=TrunkLayout0(), project="",
-            push_metadata=True, root_action=("open", ),
+            push_metadata=True, root_action=("open",  self.to_repo.get_latest_revnum()),
             base_foreign_info=self.interrepo._get_foreign_revision_info(
                 rev_merged.parent_ids[0]))
         log = self.client_log(self.svn_repo_url, 4, 0)
@@ -1494,6 +1511,8 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         config = self.interrepo._get_branch_config("trunk")
         self.addCleanup(self.from_repo.unlock)
         self.from_repo.lock_read()
+        self.to_repo.lock_write()
+        self.addCleanup(self.to_repo.unlock)
         rev1 = self.from_repo.get_revision(self.revid1)
         self.interrepo.push_single_revision("trunk",
             config, rev1, push_metadata=True, root_action=("create", ),
@@ -1502,7 +1521,7 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         self.interrepo.push_revision_inclusive("trunk", config,
             rev_merged, push_merged=True,
             layout=TrunkLayout0(), project="",
-            push_metadata=True, root_action=("open", ),
+            push_metadata=True, root_action=("open",  self.to_repo.get_latest_revnum()),
             base_foreign_info=self.interrepo._get_foreign_revision_info(rev_merged.parent_ids[0]))
 
         revid3 = self.tree2.commit('msg2 again')
@@ -1514,7 +1533,7 @@ class PushRevisionInclusiveTests(InterToSvnRepositoryTestCase):
         self.interrepo.push_revision_inclusive("trunk", config,
             rev_merged, push_merged=True,
             layout=TrunkLayout0(), project="",
-            push_metadata=True, root_action=("open",),
+            push_metadata=True, root_action=("open",  self.to_repo.get_latest_revnum()),
             base_foreign_info=self.interrepo._get_foreign_revision_info(rev_merged.parent_ids[0]))
 
         log = self.client_log(self.svn_repo_url, 6, 4)
@@ -1528,7 +1547,7 @@ class CreateBranchContainerTests(SubversionTestCase):
 
     def setUp(self):
         super(CreateBranchContainerTests, self).setUp()
-        self.repos_url = self.make_repository("d")
+        self.repos_url = self.make_svn_repository("d")
         self.repo = Repository.open(self.repos_url)
 
     def test_create_single(self):
